"""
Potential scenarios:

- Running from a Windows native "cmd"
  - Targeting Windows native (os.subsystem = None)
    - No need of bash (no conf at all)
    - Need to build in bash (tools.microsoft.bash:subsystem=xxx,
                             tools.microsoft.bash:path=<path>,
                             conanfile.win_bash)
    - Need to run (tests) in bash (tools.microsoft.bash:subsystem=xxx,
                                   tools.microsoft.bash:path=<path>,
                                   conanfile.win_bash_run)
  - Targeting Subsystem (os.subsystem = msys2/cygwin)
    - Always builds and runs in bash (tools.microsoft.bash:path)

- Running from a subsytem terminal (tools.microsoft.bash:subsystem=xxx,
                                    tools.microsoft.bash:path=None) NO ERROR mode for not specifying it? =CURRENT?
  - Targeting Windows native (os.subsystem = None)
  - Targeting Subsystem (os.subsystem = msys2/cygwin)

"""
import os
import platform
import re
import subprocess

from conans.errors import ConanException

WINDOWS = "windows"
MSYS2 = 'msys2'
MSYS = 'msys'
CYGWIN = 'cygwin'
WSL = 'wsl'  # Windows Subsystem for Linux
SFU = 'sfu'  # Windows Services for UNIX


def command_env_wrapper(conanfile, command, envfiles, envfiles_folder):
    from conan.tools.env.environment import environment_wrap_command
    if platform.system() == "Windows" and conanfile.win_bash:  # New, Conan 2.0
        wrapped_cmd = _windows_bash_wrapper(conanfile, command, envfiles, envfiles_folder)
    else:
        wrapped_cmd = environment_wrap_command(envfiles, envfiles_folder, command)
    return wrapped_cmd


def _windows_bash_wrapper(conanfile, command, env, envfiles_folder):
    from conan.tools.env import Environment
    from conan.tools.env.environment import environment_wrap_command
    """ Will wrap a unix command inside a bash terminal It requires to have MSYS2, CYGWIN, or WSL"""

    subsystem = conanfile.conf.get("tools.microsoft.bash:subsystem")
    shell_path = conanfile.conf.get("tools.microsoft.bash:path")
    if not subsystem or not shell_path:
        raise ConanException("The config 'tools.microsoft.bash:subsystem' and "
                             "'tools.microsoft.bash:path' are "
                             "needed to run commands in a Windows subsystem")
    env = env or []
    if subsystem == MSYS2:
        # Configure MSYS2 to inherith the PATH
        msys2_mode_env = Environment()
        _msystem = {"x86": "MINGW32"}.get(conanfile.settings.get_safe("arch"), "MINGW64")
        msys2_mode_env.define("MSYSTEM", _msystem)
        msys2_mode_env.define("MSYS2_PATH_TYPE", "inherit")
        path = os.path.join(conanfile.generators_folder, "msys2_mode.bat")
        msys2_mode_env.vars(conanfile, "build").save_bat(path)
        env.append(path)

    wrapped_shell = '"%s"' % shell_path if " " in shell_path else shell_path
    wrapped_shell = environment_wrap_command(env, envfiles_folder, wrapped_shell,
                                             accepted_extensions=("bat", "ps1"))

    # Wrapping the inside_command enable to prioritize our environment, otherwise /usr/bin go
    # first and there could be commands that we want to skip
    wrapped_user_cmd = environment_wrap_command(env, envfiles_folder, command,
                                                accepted_extensions=("sh", ))
    wrapped_user_cmd = _escape_windows_cmd(wrapped_user_cmd)

    final_command = '{} -c {}'.format(wrapped_shell, wrapped_user_cmd)
    return final_command


def _escape_windows_cmd(command):
    """ To use in a regular windows cmd.exe
        1. Adds escapes so the argument can be unpacked by CommandLineToArgvW()
        2. Adds escapes for cmd.exe so the argument survives cmd.exe's substitutions.

        Useful to escape commands to be executed in a windows bash (msys2, cygwin etc)
    """
    quoted_arg = subprocess.list2cmdline([command])
    return "".join(["^%s" % arg if arg in r'()%!^"<>&|' else arg for arg in quoted_arg])


def deduce_subsystem(conanfile, scope):
    """ used by:
    - EnvVars: to decide if using :  ; as path separator, translate paths to subsystem
               and decide to generate a .bat or .sh
    - Autotools: to define the full abs path to the "configure" script
    - GnuDeps: to map all the paths from dependencies
    - Aggregation of envfiles: to map each aggregated path to the subsystem
    - unix_path: util for recipes
    """
    if scope.startswith("build"):
        if hasattr(conanfile, "settings_build"):
            the_os = conanfile.settings_build.get_safe("os")
            subsystem = conanfile.settings_build.get_safe("os.subsystem")
        else:
            the_os = platform.system()  # FIXME: Temporary fallback until 2.0
            subsystem = None
    else:
        the_os = conanfile.settings.get_safe("os")
        subsystem = conanfile.settings.get_safe("os.subsystem")

    if not str(the_os).startswith("Windows"):
        return None

    if subsystem is None and not scope.startswith("build"):  # "run" scope do not follow win_bash
        return WINDOWS

    if subsystem is None:  # Not defined by settings, so native windows
        if not conanfile.win_bash:
            return WINDOWS

        subsystem = conanfile.conf.get("tools.microsoft.bash:subsystem")
        if not subsystem:
            raise ConanException("The config 'tools.microsoft.bash:subsystem' is "
                                 "needed to run commands in a Windows subsystem")
    return subsystem


def subsystem_path(subsystem, path):
    """"Used to translate windows paths to MSYS unix paths like
    c/users/path/to/file. Not working in a regular console or MinGW!
    """
    if subsystem is None or subsystem == WINDOWS:
        return path

    if os.path.exists(path):
        # if the path doesn't exist (and abs) we cannot guess the casing
        path = get_cased_path(path)

    if path.startswith('\\\\?\\'):
        path = path[4:]
    path = path.replace(":/", ":\\")
    append_prefix = re.match(r'[a-z]:\\', path, re.IGNORECASE)
    pattern = re.compile(r'([a-z]):\\', re.IGNORECASE)
    path = pattern.sub('/\\1/', path).replace('\\', '/')

    if append_prefix:
        if subsystem in (MSYS, MSYS2):
            return path.lower()
        elif subsystem == CYGWIN:
            return '/cygdrive' + path.lower()
        elif subsystem == WSL:
            return '/mnt' + path[0:2].lower() + path[2:]
        elif subsystem == SFU:
            path = path.lower()
            return '/dev/fs' + path[0] + path[1:].capitalize()
    else:
        return path if subsystem == WSL else path.lower()
    return None


def get_cased_path(name):
    if platform.system() != "Windows":
        return name
    if not os.path.isabs(name):
        name = os.path.abspath(name)

    result = []
    current = name
    while True:
        parent, child = os.path.split(current)
        if parent == current:
            break

        child_cased = child
        if os.path.exists(parent):
            children = os.listdir(parent)
            for c in children:
                if c.upper() == child.upper():
                    child_cased = c
                    break
        result.append(child_cased)
        current = parent
    drive, _ = os.path.splitdrive(current)
    result.append(drive)
    return os.sep.join(reversed(result))



# coding=utf-8

import unittest

from conans.model.build_info import _CppInfo


class CppFlagsTest(unittest.TestCase):
    """ Test that deprecated 'cppflags' still works (keep backwards compatibility) """

    def test_use_cxxflags(self):
        """ Changes in cxxflags get reflected in cppflags """
        cpp_info = _CppInfo()
        cpp_info.cxxflags = "flags"
        self.assertEqual(cpp_info.cppflags, "flags")
        self.assertEqual(cpp_info.cxxflags, cpp_info.cppflags)

        cpp_info.cxxflags = None
        self.assertEqual(cpp_info.cppflags, None)
        self.assertEqual(cpp_info.cxxflags, cpp_info.cppflags)

    def test_use_cppflags(self):
        """ Changes in cppflags get reflected in cxxflags """
        cpp_info = _CppInfo()

        cpp_info.cppflags = "flags"
        self.assertEqual(cpp_info.cxxflags, "flags")
        self.assertEqual(cpp_info.cxxflags, cpp_info.cppflags)

        cpp_info.cppflags = None
        self.assertEqual(cpp_info.cxxflags, None)
        self.assertEqual(cpp_info.cxxflags, cpp_info.cppflags)



import textwrap

lib_h = textwrap.dedent("""
    #pragma once
    #include <string>
    class HelloLib {
    public:
        void hello(const std::string& name);
    };
""")

lib_cpp = textwrap.dedent("""
    #include "hello.h"
    #include <iostream>
    using namespace std;
    void HelloLib::hello(const std::string& name) {
        #ifdef DEBUG
        std::cout << "Hello " << name << " Debug!" <<std::endl;
        #else
        std::cout << "Hello " << name << " Release!" <<std::endl;
        #endif
    }
""")

cmakelists = textwrap.dedent("""
    cmake_minimum_required(VERSION 3.1)
    project(MyHello CXX)
    set(SOURCES
      hello.cpp
   )
    set(HEADERS
        hello.h
    )
    add_library (hello ${SOURCES} ${HEADERS})
    set_target_properties(hello PROPERTIES PUBLIC_HEADER "${HEADERS}")
    install(TARGETS hello
        RUNTIME DESTINATION bin
        LIBRARY DESTINATION lib
        ARCHIVE DESTINATION lib
        PUBLIC_HEADER DESTINATION include
    )
""")


def create_library(client):
    client.save({
        'hello.h': lib_h,
        'hello.cpp': lib_cpp,
        'CMakeLists.txt': cmakelists
    })



import os
import platform
import textwrap

import pytest
import six
from mock import patch

from conans.errors import ConanException
from conans.util.files import save
from conans.test.utils.tools import TestClient


@pytest.fixture
def client():
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conans import ConanFile

        class Pkg(ConanFile):
            def generate(self):
                for k, v in self.conf.items():
                    self.output.info("{}${}".format(k, v))
        """)
    client.save({"conanfile.py": conanfile})
    return client


def test_basic_composition(client):
    profile1 = textwrap.dedent("""\
        [conf]
        tools.microsoft.msbuild:verbosity=Quiet
        tools.microsoft.msbuild:performance=Slow
        tools.cmake.cmake:verbosity=Extra
        """)
    profile2 = textwrap.dedent("""\
        [conf]
        tools.microsoft.msbuild:verbosity=Minimal
        tools.microsoft.msbuild:robustness=High
        tools.meson.meson:verbosity=Super
        """)
    client.save({"profile1": profile1,
                 "profile2": profile2})
    client.run("install . -pr=profile1")
    assert "tools.microsoft.msbuild:verbosity$Quiet" in client.out
    assert "tools.microsoft.msbuild:performance$Slow" in client.out
    assert "tools.cmake.cmake:verbosity$Extra" in client.out

    client.run("install . -pr=profile1 -pr=profile2")
    assert "tools.microsoft.msbuild:verbosity$Minimal" in client.out
    assert "tools.microsoft.msbuild:performance$Slow" in client.out
    assert "tools.microsoft.msbuild:robustness$High" in client.out
    assert "tools.cmake.cmake:verbosity$Extra" in client.out
    assert "tools.meson.meson:verbosity$Super" in client.out

    client.run("install . -pr=profile2 -pr=profile1")
    assert "tools.microsoft.msbuild:verbosity$Quiet" in client.out
    assert "tools.microsoft.msbuild:performance$Slow" in client.out
    assert "tools.microsoft.msbuild:robustness$High" in client.out
    assert "tools.cmake.cmake:verbosity$Extra" in client.out
    assert "tools.meson.meson:verbosity$Super" in client.out


def test_basic_inclusion(client):
    profile1 = textwrap.dedent("""\
        [conf]
        tools.microsoft.msbuild:verbosity=Quiet
        tools.microsoft.msbuild:performance=Slow
        tools.cmake.cmake:verbosity=Extra
        """)
    profile2 = textwrap.dedent("""\
        include(profile1)
        [conf]
        tools.microsoft.msbuild:verbosity=Minimal
        tools.microsoft.msbuild:robustness=High
        tools.meson.meson:verbosity=Super
        """)
    client.save({"profile1": profile1,
                 "profile2": profile2})

    client.run("install . -pr=profile2")
    assert "tools.microsoft.msbuild:verbosity$Minimal" in client.out
    assert "tools.microsoft.msbuild:performance$Slow" in client.out
    assert "tools.microsoft.msbuild:robustness$High" in client.out
    assert "tools.cmake.cmake:verbosity$Extra" in client.out
    assert "tools.meson.meson:verbosity$Super" in client.out


def test_composition_conan_conf(client):
    conf = textwrap.dedent("""\
        tools.microsoft.msbuild:verbosity=Quiet
        tools.microsoft.msbuild:performance=Slow
        tools.cmake.cmake:verbosity=Extra
        """)
    save(client.cache.new_config_path, conf)
    profile = textwrap.dedent("""\
        [conf]
        tools.microsoft.msbuild:verbosity=Minimal
        tools.microsoft.msbuild:robustness=High
        tools.meson.meson:verbosity=Super
        """)
    client.save({"profile": profile})
    client.run("install . -pr=profile")
    assert "tools.microsoft.msbuild:verbosity$Minimal" in client.out
    assert "tools.microsoft.msbuild:performance$Slow" in client.out
    assert "tools.microsoft.msbuild:robustness$High" in client.out
    assert "tools.cmake.cmake:verbosity$Extra" in client.out
    assert "tools.meson.meson:verbosity$Super" in client.out


def test_new_config_file(client):
    conf = textwrap.dedent("""\
        tools.microsoft.msbuild:verbosity=Minimal
        user.mycompany.myhelper:myconfig=myvalue
        *:tools.cmake.cmake:generator=X
        cache:no_locks=True
        cache:read_only=True
        """)
    save(client.cache.new_config_path, conf)
    client.run("install .")
    assert "tools.microsoft.msbuild:verbosity$Minimal" in client.out
    assert "user.mycompany.myhelper:myconfig$myvalue" in client.out
    assert "tools.cmake.cmake:generator$X" in client.out
    assert "no_locks" not in client.out
    assert "read_only" not in client.out


@patch("conans.client.conf.required_version.client_version", "1.26.0")
def test_new_config_file_required_version():
    client = TestClient()
    conf = textwrap.dedent("""\
        core:required_conan_version=>=2.0
        """)
    save(client.cache.new_config_path, conf)
    with pytest.raises(ConanException) as excinfo:
        client.run("install .")
    assert ("Current Conan version (1.26.0) does not satisfy the defined one (>=2.0)"
            in str(excinfo.value))


def test_composition_conan_conf_overwritten_by_cli_arg(client):
    conf = textwrap.dedent("""\
        tools.microsoft.msbuild:verbosity=Quiet
        tools.microsoft.msbuild:performance=Slow
        """)
    save(client.cache.new_config_path, conf)
    profile = textwrap.dedent("""\
        [conf]
        tools.microsoft.msbuild:verbosity=Minimal
        tools.microsoft.msbuild:robustness=High
        """)
    client.save({"profile": profile})
    client.run("install . -pr=profile -c tools.microsoft.msbuild:verbosity=Detailed "
               "-c tools.meson.meson:verbosity=Super")
    assert "tools.microsoft.msbuild:verbosity$Detailed" in client.out
    assert "tools.microsoft.msbuild:performance$Slow" in client.out
    assert "tools.microsoft.msbuild:robustness$High" in client.out
    assert "tools.meson.meson:verbosity$Super" in client.out


def test_composition_conan_conf_different_data_types_by_cli_arg(client):
    """
    Testing if you want to introduce a list/dict via cli

    >> conan install . -c "tools.build.flags:ccflags+=['-Werror']"
    >> conan install . -c "tools.microsoft.msbuildtoolchain:compile_options={'ExceptionHandling': 'Async'}"

    """
    conf = textwrap.dedent("""\
        tools.build.flags:ccflags=["-Wall"]
        """)
    save(client.cache.new_config_path, conf)
    client.run('install . -c "tools.build.flags:ccflags+=[\'-Werror\']" '
               '-c "tools.microsoft.msbuildtoolchain:compile_options={\'ExceptionHandling\': \'Async\'}"')

    assert "tools.build.flags:ccflags$['-Wall', '-Werror']" in client.out
    assert "tools.microsoft.msbuildtoolchain:compile_options${'ExceptionHandling': 'Async'}" in client.out


@pytest.mark.skipif(six.PY2, reason="only Py3")
def test_jinja_global_conf(client):
    save(client.cache.new_config_path, "user.mycompany:parallel = {{os.cpu_count()/2}}\n"
                                       "user.mycompany:other = {{platform.system()}}\n")
    client.run("install .")
    assert "user.mycompany:parallel={}".format(os.cpu_count()/2) in client.out
    assert "user.mycompany:other={}".format(platform.system()) in client.out



import os
import zipfile
from os.path import basename

import pytest

from conan.tools.files import unzip
from conans.test.utils.mocks import MockConanfile
from conans.test.utils.test_files import temp_folder
from conans.util.files import save


def test_impossible_to_import_untargz():
    with pytest.raises(ImportError) as exc:
        from conan.tools.files import untargz


def test_unzip():
    tmp_dir = temp_folder()
    file_path = os.path.join(tmp_dir, "foo.txt")
    save(os.path.join(tmp_dir, "foo.txt"), "bar")
    zf = zipfile.ZipFile(os.path.join(tmp_dir, 'zipfile.zip'), mode='w')
    zf.write(file_path, basename(file_path))
    zf.close()

    conanfile = MockConanfile({})

    # Unzip and check permissions are kept
    dest_dir = temp_folder()
    unzip(conanfile, os.path.join(tmp_dir, 'zipfile.zip'), dest_dir)
    assert os.path.exists(os.path.join(dest_dir, "foo.txt"))



import platform
import textwrap

import pytest

from conans.test.utils.tools import TestClient


ios10_armv8_settings = "-s os=iOS -s os.sdk=iphoneos -s os.version=10.0 -s arch=armv8"


class _FindRootPathModes(object):
    def __init__(self, package=None, library=None, framework=None, include=None, program=None):
        self.package = package
        self.library = library
        self.framework = framework
        self.include = include
        self.program = program


find_root_path_modes_default = _FindRootPathModes()
find_root_path_modes_cross_build = _FindRootPathModes(
    package="ONLY",
    library="ONLY",
    framework="ONLY",
    include="ONLY",
    program="NEVER",
)


def _cmake_command_toolchain(find_root_path_modes):
    build_type = "-DCMAKE_BUILD_TYPE=Release" if platform.system() != "Windows" else ""
    cmake_command = "cmake .. -DCMAKE_TOOLCHAIN_FILE=../conan_toolchain.cmake {}".format(build_type)
    if find_root_path_modes.package:
        cmake_command += " -DCMAKE_FIND_ROOT_PATH_MODE_PACKAGE={}".format(find_root_path_modes.package)
    if find_root_path_modes.library:
        cmake_command += " -DCMAKE_FIND_ROOT_PATH_MODE_LIBRARY={}".format(find_root_path_modes.library)
    if find_root_path_modes.framework:
        cmake_command += " -DCMAKE_FIND_ROOT_PATH_MODE_FRAMEWORK={}".format(find_root_path_modes.framework)
    if find_root_path_modes.include:
        cmake_command += " -DCMAKE_FIND_ROOT_PATH_MODE_INCLUDE={}".format(find_root_path_modes.include)
    if find_root_path_modes.program:
        cmake_command += " -DCMAKE_FIND_ROOT_PATH_MODE_PROGRAM={}".format(find_root_path_modes.program)
    return cmake_command


@pytest.mark.tool_cmake
@pytest.mark.parametrize("package", ["hello", "ZLIB"])
@pytest.mark.parametrize("find_package", ["module", "config"])
@pytest.mark.parametrize(
    "settings",
    [
        "",
        pytest.param(
            ios10_armv8_settings,
            marks=pytest.mark.skipif(platform.system() != "Darwin", reason="OSX only"),
        ),
    ],
)
@pytest.mark.parametrize(
    "find_root_path_modes", [find_root_path_modes_default, find_root_path_modes_cross_build],
)
def test_cmaketoolchain_path_find_package(package, find_package, settings, find_root_path_modes):
    """Test with user "Hello" and also ZLIB one, to check that package ZLIB
    has priority over the CMake system one
    """
    client = TestClient()

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class TestConan(ConanFile):
            exports_sources = "*"
            def layout(self):
                pass
            def package(self):
                self.copy(pattern="*")
            def package_info(self):
                self.cpp_info.builddirs.append("cmake")
        """)
    find = textwrap.dedent("""
        SET({package}_FOUND 1)
        MESSAGE("HELLO FROM THE {package} FIND PACKAGE!")
        """).format(package=package)

    filename = "{}Config.cmake" if find_package == "config" else "Find{}.cmake"
    filename = filename.format(package)
    client.save({"conanfile.py": conanfile, "cmake/{}".format(filename): find})
    client.run("create . {}/0.1@ -pr:b default {}".format(package, settings))

    consumer = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(MyHello CXX)
        find_package({package} REQUIRED)
        """).format(package=package)

    client.save({"CMakeLists.txt": consumer}, clean_first=True)
    client.run("install {}/0.1@ -g CMakeToolchain -pr:b default {}".format(package, settings))
    with client.chdir("build"):
        client.run_command(_cmake_command_toolchain(find_root_path_modes))
    assert "Conan: Target declared" not in client.out
    assert "HELLO FROM THE {package} FIND PACKAGE!".format(package=package) in client.out

    # If using the CMakeDeps generator, the in-package .cmake will be ignored
    client.run("install {}/0.1@ -g CMakeToolchain -g CMakeDeps -pr:b default {}".format(
        package, settings,
    ))
    with client.chdir("build2"):  # A clean folder, not the previous one, CMake cache doesnt affect
        client.run_command(_cmake_command_toolchain(find_root_path_modes))
    assert "Conan: Target declared '{package}::{package}'".format(package=package) in client.out
    assert "HELLO FROM THE {package} FIND PACKAGE!".format(package=package) not in client.out


def test_cmaketoolchain_path_find_package_editable():
    """ make sure a package in editable mode that contains a xxxConfig.cmake file can find that
    file in the user folder
    """
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import cmake_layout
        class TestConan(ConanFile):
            settings = "os", "compiler", "arch", "build_type"
            exports_sources = "*"
            def layout(self):
                cmake_layout(self)
                self.cpp.source.builddirs = ["cmake"]
            def package(self):
                self.copy(pattern="*")
            def package_info(self):
                self.cpp_info.builddirs.append("cmake")
        """)
    find = textwrap.dedent("""
        SET(hello_FOUND 1)
        MESSAGE("HELLO FROM THE hello FIND PACKAGE!")
        """)

    consumer = textwrap.dedent("""\
        cmake_minimum_required(VERSION 3.15)
        project(MyHello CXX)
        find_package(hello REQUIRED)
        """)
    client.save({"dep/conanfile.py": conanfile,
                 "dep/cmake/helloConfig.cmake": find,
                 "consumer/conanfile.txt": "[requires]\nhello/0.1\n[generators]\nCMakeToolchain",
                 "consumer/CMakeLists.txt": consumer})
    with client.chdir("dep"):
        client.run("install .")
        client.run("editable add . hello/0.1@")

    with client.chdir("consumer"):
        client.run("install .")

        with client.chdir("build"):
            build_type = "-DCMAKE_BUILD_TYPE=Release" if platform.system() != "Windows" else ""
            cmake_command = "cmake .. -DCMAKE_TOOLCHAIN_FILE=../conan_toolchain.cmake {}".format(
                build_type)
            client.run_command(cmake_command)
        assert "Conan: Target declared" not in client.out
        assert "HELLO FROM THE hello FIND PACKAGE!" in client.out


@pytest.mark.tool_cmake
@pytest.mark.parametrize(
    "settings",
    [
        "",
        pytest.param(
            ios10_armv8_settings,
            marks=pytest.mark.skipif(platform.system() != "Darwin", reason="OSX only"),
        ),
    ],
)
@pytest.mark.parametrize(
    "find_root_path_modes", [find_root_path_modes_default, find_root_path_modes_cross_build],
)
def test_cmaketoolchain_path_find_package_real_config(settings, find_root_path_modes):
    client = TestClient()

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.cmake import CMake
        import os
        class TestConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            exports_sources = "*"
            generators = "CMakeToolchain"

            def layout(self):
                pass

            def build(self):
                cmake = CMake(self)
                cmake.configure()

            def package(self):
                cmake = CMake(self)
                cmake.install()

            def package_info(self):
                self.cpp_info.builddirs.append(os.path.join("hello", "cmake"))
        """)
    cmake = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(MyHello NONE)

        add_library(hello INTERFACE)
        install(TARGETS hello EXPORT helloConfig)
        export(TARGETS hello
            NAMESPACE hello::
            FILE "${CMAKE_CURRENT_BINARY_DIR}/helloConfig.cmake"
        )
        install(EXPORT helloConfig
            DESTINATION "${CMAKE_INSTALL_PREFIX}/hello/cmake"
            NAMESPACE hello::
        )
        """)
    client.save({"conanfile.py": conanfile, "CMakeLists.txt": cmake})
    client.run("create . hello/0.1@ -pr:b default {}".format(settings))

    consumer = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(MyHello NONE)

        find_package(hello REQUIRED)
        """)

    client.save({"CMakeLists.txt": consumer}, clean_first=True)
    client.run("install hello/0.1@ -g CMakeToolchain -pr:b default {}".format(settings))
    with client.chdir("build"):
        client.run_command(_cmake_command_toolchain(find_root_path_modes))
    # If it didn't fail, it found the helloConfig.cmake
    assert "Conan: Target declared" not in client.out

    # If using the CMakeDeps generator, the in-package .cmake will be ignored
    client.run("install hello/0.1@ -g CMakeToolchain -g CMakeDeps -pr:b default {}".format(settings))
    with client.chdir("build2"):  # A clean folder, not the previous one, CMake cache doesnt affect
        client.run_command(_cmake_command_toolchain(find_root_path_modes))
    assert "Conan: Target declared 'hello::hello'" in client.out


@pytest.mark.tool_cmake
@pytest.mark.parametrize("require_type", ["requires", "build_requires"])
@pytest.mark.parametrize(
    "settings",
    [
        "",
        pytest.param(
            ios10_armv8_settings,
            marks=pytest.mark.skipif(platform.system() != "Darwin", reason="OSX only"),
        ),
    ],
)
@pytest.mark.parametrize(
    "find_root_path_modes", [find_root_path_modes_default, find_root_path_modes_cross_build],
)
def test_cmaketoolchain_path_include_cmake_modules(require_type, settings, find_root_path_modes):
    """Test that cmake module files in builddirs of requires and build_requires
    are accessible with include() in consumer CMakeLists
    """
    client = TestClient()

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class TestConan(ConanFile):
            settings = "os", "compiler", "arch", "build_type"
            exports_sources = "*"
            def layout(self):
                pass
            def package(self):
                self.copy(pattern="*")
            def package_info(self):
                self.cpp_info.builddirs.append("cmake")
    """)
    myowncmake = 'MESSAGE("MYOWNCMAKE FROM hello!")'
    client.save({"conanfile.py": conanfile, "cmake/myowncmake.cmake": myowncmake})
    client.run("create . hello/0.1@{}".format(
        " -pr:b default {}".format(settings) if require_type == "requires" else "",
    ))

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class PkgConan(ConanFile):
            settings = "os", "compiler", "arch", "build_type"
            {require_type} = "hello/0.1"
    """.format(require_type=require_type))
    consumer = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(MyHello)
        include(myowncmake)
    """)
    client.save({"conanfile.py": conanfile, "CMakeLists.txt": consumer}, clean_first=True)
    client.run("install . pkg/0.1@ -g CMakeToolchain -pr:b default {}".format(settings))
    with client.chdir("build"):
        client.run_command(_cmake_command_toolchain(find_root_path_modes))
    assert "MYOWNCMAKE FROM hello!" in client.out


@pytest.mark.tool_cmake
@pytest.mark.parametrize(
    "settings",
    [
        "",
        pytest.param(
            ios10_armv8_settings,
            marks=pytest.mark.skipif(platform.system() != "Darwin", reason="OSX only"),
        ),
    ],
)
@pytest.mark.parametrize(
    "find_root_path_modes", [find_root_path_modes_default, find_root_path_modes_cross_build],
)
def test_cmaketoolchain_path_find_file_find_path(settings, find_root_path_modes):
    """Test that headers in includedirs of requires can be found with
    find_file() and find_path() in consumer CMakeLists
    """
    client = TestClient()

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class TestConan(ConanFile):
            settings = "os", "arch", "compiler", "build_type"
            exports_sources = "*"
            def layout(self):
                pass
            def package(self):
                self.copy(pattern="*.h", dst="include")
    """)
    client.save({"conanfile.py": conanfile, "hello.h": ""})
    client.run("create . hello/0.1@ -pr:b default {}".format(settings))

    consumer = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(MyHello)
        find_file(HELLOFILE hello.h)
        if(HELLOFILE)
            message("Found file hello.h")
        endif()
        find_path(HELLODIR hello.h)
        if(HELLODIR)
            message("Found path of hello.h")
        endif()
    """)
    client.save({"CMakeLists.txt": consumer}, clean_first=True)
    client.run("install hello/0.1@ -g CMakeToolchain -pr:b default {}".format(settings))
    with client.chdir("build"):
        client.run_command(_cmake_command_toolchain(find_root_path_modes))
    assert "Found file hello.h" in client.out
    assert "Found path of hello.h" in client.out


@pytest.mark.tool_cmake
@pytest.mark.parametrize(
    "settings",
    [
        "",
        pytest.param(
            ios10_armv8_settings,
            marks=pytest.mark.skipif(platform.system() != "Darwin", reason="OSX only"),
        ),
    ],
)
@pytest.mark.parametrize(
    "find_root_path_modes", [find_root_path_modes_default, find_root_path_modes_cross_build],
)
def test_cmaketoolchain_path_find_library(settings, find_root_path_modes):
    """Test that libraries in libdirs of requires can be found with
    find_library() in consumer CMakeLists
    """
    client = TestClient()

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class TestConan(ConanFile):
            settings = "os", "arch", "compiler", "build_type"
            exports_sources = "*"
            def layout(self):
                pass
            def package(self):
                self.copy(pattern="*", dst="lib")
    """)
    client.save({"conanfile.py": conanfile, "libhello.a": "", "hello.lib": ""})
    client.run("create . hello_host/0.1@ -pr:b default {}".format(settings))
    client.run("create . hello_build/0.1@")

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class PkgConan(ConanFile):
            settings = "os", "arch", "compiler", "build_type"
            requires = "hello_host/0.1"
            build_requires = "hello_build/0.1"
    """)
    consumer = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(MyHello)
        find_library(HELLOLIB hello)
        if(HELLOLIB)
            message("Found hello lib: ${HELLOLIB}")
        endif()
    """)
    client.save({"conanfile.py": conanfile, "CMakeLists.txt": consumer}, clean_first=True)
    client.run("install . pkg/0.1@ -g CMakeToolchain -pr:b default {}".format(settings))
    with client.chdir("build"):
        client.run_command(_cmake_command_toolchain(find_root_path_modes))
    assert "Found hello lib" in client.out
    assert "hello_host/0.1/" in client.out
    assert "hello_build/0.1/" not in client.out


@pytest.mark.tool_cmake
@pytest.mark.parametrize(
    "settings",
    [
        "",
        pytest.param(
            ios10_armv8_settings,
            marks=pytest.mark.skipif(platform.system() != "Darwin", reason="OSX only"),
        ),
    ],
)
@pytest.mark.parametrize(
    "find_root_path_modes", [find_root_path_modes_default, find_root_path_modes_cross_build],
)
def test_cmaketoolchain_path_find_program(settings, find_root_path_modes):
    """Test that executables in bindirs of build_requires can be found with
    find_program() in consumer CMakeLists.
    """
    client = TestClient()

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class TestConan(ConanFile):
            settings = "os", "arch", "compiler", "build_type"
            exports_sources = "*"
            def layout(self):
                pass
            def package(self):
                self.copy(pattern="*", dst="bin")
    """)
    client.save({"conanfile.py": conanfile, "hello": "", "hello.exe": ""})
    client.run("create . hello_host/0.1@ -pr:b default {}".format(settings))
    client.run("create . hello_build/0.1@")

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class PkgConan(ConanFile):
            settings = "os", "arch", "compiler", "build_type"
            requires = "hello_host/0.1"
            build_requires = "hello_build/0.1"
    """)
    consumer = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(MyHello)
        find_program(HELLOPROG hello)
        if(HELLOPROG)
            message("Found hello prog: ${HELLOPROG}")
        endif()
    """)
    client.save({"conanfile.py": conanfile, "CMakeLists.txt": consumer}, clean_first=True)
    client.run("install . pkg/0.1@ -g CMakeToolchain -pr:b default {}".format(settings))
    with client.chdir("build"):
        client.run_command(_cmake_command_toolchain(find_root_path_modes))
    assert "Found hello prog" in client.out
    assert "hello_host/0.1/" not in client.out
    assert "hello_build/0.1/" in client.out



import json
import os
import textwrap
import unittest
from datetime import datetime

from conans import __version__ as client_version
from conans.test.utils.tools import TestClient, GenConanfile, NO_SETTINGS_PACKAGE_ID
from conans.util.files import save, load


class InfoTest(unittest.TestCase):

    def _create(self, name, version, deps=None, export=True):
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                name = "{name}"
                version = "{version}"
                license = {license}
                description = "blah"
                url = "myurl"
                {requires}
            """)
        requires = ""
        if deps:
            requires = "requires = {}".format(", ".join('"{}"'.format(d) for d in deps))

        conanfile = conanfile.format(name=name, version=version, requires=requires,
                                     license='"MIT"')

        self.client.save({"conanfile.py": conanfile}, clean_first=True)
        if export:
            self.client.run("export . lasote/stable")

    def test_graph(self):
        self.client = TestClient()

        test_deps = {
            "Hello0": ["Hello1", "Hello2", "Hello3"],
            "Hello1": ["Hello4"],
            "Hello2": [],
            "Hello3": ["Hello7"],
            "Hello4": ["Hello5", "Hello6"],
            "Hello5": [],
            "Hello6": [],
            "Hello7": ["Hello8"],
            "Hello8": ["Hello9", "Hello10"],
            "Hello9": [],
            "Hello10": [],
        }

        def create_export(testdeps, name):
            deps = testdeps[name]
            for dep in deps:
                create_export(testdeps, dep)

            expanded_deps = ["%s/0.1@lasote/stable" % dep for dep in deps]
            export = False if name == "Hello0" else True
            self._create(name, "0.1", expanded_deps, export=export)

        create_export(test_deps, "Hello0")

        self.client.run("info . --graph", assert_error=True)

        # arbitrary case - file will be named according to argument
        arg_filename = "test.dot"
        self.client.run("info . --graph=%s" % arg_filename)
        dot_file = os.path.join(self.client.current_folder, arg_filename)
        contents = load(dot_file)
        expected = textwrap.dedent("""
            "Hello8/0.1@lasote/stable" -> "Hello9/0.1@lasote/stable"
            "Hello8/0.1@lasote/stable" -> "Hello10/0.1@lasote/stable"
            "Hello4/0.1@lasote/stable" -> "Hello5/0.1@lasote/stable"
            "Hello4/0.1@lasote/stable" -> "Hello6/0.1@lasote/stable"
            "Hello3/0.1@lasote/stable" -> "Hello7/0.1@lasote/stable"
            "Hello7/0.1@lasote/stable" -> "Hello8/0.1@lasote/stable"
            "conanfile.py (Hello0/0.1)" -> "Hello1/0.1@lasote/stable"
            "conanfile.py (Hello0/0.1)" -> "Hello2/0.1@lasote/stable"
            "conanfile.py (Hello0/0.1)" -> "Hello3/0.1@lasote/stable"
            "Hello1/0.1@lasote/stable" -> "Hello4/0.1@lasote/stable"
            """)
        for line in expected.splitlines():
            assert line in contents

    def test_graph_html(self):
        self.client = TestClient()

        test_deps = {
            "Hello0": ["Hello1"],
            "Hello1": [],
        }

        def create_export(testdeps, name):
            deps = testdeps[name]
            for dep in deps:
                create_export(testdeps, dep)

            expanded_deps = ["%s/0.1@lasote/stable" % dep for dep in deps]
            export = False if name == "Hello0" else True
            self._create(name, "0.1", expanded_deps, export=export)

        create_export(test_deps, "Hello0")

        # arbitrary case - file will be named according to argument
        arg_filename = "test.html"
        self.client.run("info . --graph=%s" % arg_filename)
        html = self.client.load(arg_filename)
        self.assertIn("<body>", html)
        self.assertIn("{ from: 0, to: 1 }", html)
        self.assertIn("id: 0,\n                        label: 'Hello0/0.1',", html)
        self.assertIn("Conan <b>v{}</b> <script>document.write(new Date().getFullYear())</script>"
                      " JFrog LTD. <a>https://conan.io</a>"
                      .format(client_version, datetime.today().year), html)

    def test_only_names(self):
        self.client = TestClient()
        self._create("Hello0", "0.1")
        self._create("Hello1", "0.1", ["Hello0/0.1@lasote/stable"])
        self._create("Hello2", "0.1", ["Hello1/0.1@lasote/stable"], export=False)

        self.client.run("info . --only None")
        self.assertEqual(["Hello0/0.1@lasote/stable", "Hello1/0.1@lasote/stable",
                          "conanfile.py (Hello2/0.1)"],
                         str(self.client.out).splitlines()[-3:])
        self.client.run("info . --only=date")
        lines = [(line if "date" not in line else "Date")
                 for line in str(self.client.out).splitlines()]
        self.assertEqual(["Hello0/0.1@lasote/stable", "Date",
                          "Hello1/0.1@lasote/stable", "Date",
                          "conanfile.py (Hello2/0.1)"], lines)

        self.client.run("info . --only=invalid", assert_error=True)
        self.assertIn("Invalid --only value", self.client.out)
        self.assertNotIn("with --path specified, allowed values:", self.client.out)

        self.client.run("info . --paths --only=bad", assert_error=True)
        self.assertIn("Invalid --only value", self.client.out)
        self.assertIn("with --path specified, allowed values:", self.client.out)

    def test_info_virtual(self):
        # Checking that "Required by: virtual" doesnt appear in the output
        self.client = TestClient()
        self._create("Hello", "0.1")
        self.client.run("info Hello/0.1@lasote/stable")
        self.assertNotIn("virtual", self.client.out)
        self.assertNotIn("Required", self.client.out)

    def test_reuse(self):
        self.client = TestClient()
        self._create("Hello0", "0.1")
        self._create("Hello1", "0.1", ["Hello0/0.1@lasote/stable"])
        self._create("Hello2", "0.1", ["Hello1/0.1@lasote/stable"], export=False)

        self.client.run("info . -u")

        self.assertIn("Creation date: ", self.client.out)
        self.assertIn("ID: ", self.client.out)
        self.assertIn("BuildID: ", self.client.out)

        expected_output = textwrap.dedent("""\
            Hello0/0.1@lasote/stable
                Remote: None
                URL: myurl
                License: MIT
                Description: blah
                Provides: Hello0
                Recipe: No remote%s
                Binary: Missing
                Binary remote: None
                Required by:
                    Hello1/0.1@lasote/stable
            Hello1/0.1@lasote/stable
                Remote: None
                URL: myurl
                License: MIT
                Description: blah
                Provides: Hello1
                Recipe: No remote%s
                Binary: Missing
                Binary remote: None
                Required by:
                    conanfile.py (Hello2/0.1)
                Requires:
                    Hello0/0.1@lasote/stable
            conanfile.py (Hello2/0.1)
                URL: myurl
                License: MIT
                Description: blah
                Provides: Hello2
                Requires:
                    Hello1/0.1@lasote/stable""")

        expected_output = expected_output % (
            "\n    Revision: d6727bc577b5c6bd8ac7261eff98be93"
            "\n    Package revision: None",
            "\n    Revision: 7c5e142433a3ee0acaeffb4454a6d42f"
            "\n    Package revision: None",) \
            if self.client.cache.config.revisions_enabled else expected_output % ("", "")

        def clean_output(output):
            return "\n".join([line for line in str(output).splitlines()
                              if not line.strip().startswith("Creation date") and
                              not line.strip().startswith("ID") and
                              not line.strip().startswith("Context") and
                              not line.strip().startswith("BuildID") and
                              not line.strip().startswith("export_folder") and
                              not line.strip().startswith("build_folder") and
                              not line.strip().startswith("source_folder") and
                              not line.strip().startswith("package_folder")])

        # The timestamp is variable so we can't check the equality
        self.assertIn(expected_output, clean_output(self.client.out))

        self.client.run("info . -u --only=url")
        expected_output = textwrap.dedent("""\
            Hello0/0.1@lasote/stable
                URL: myurl
            Hello1/0.1@lasote/stable
                URL: myurl
            conanfile.py (Hello2/0.1)
                URL: myurl""")

        self.assertIn(expected_output, clean_output(self.client.out))
        self.client.run("info . -u --only=url --only=license")
        expected_output = textwrap.dedent("""\
            Hello0/0.1@lasote/stable
                URL: myurl
                License: MIT
            Hello1/0.1@lasote/stable
                URL: myurl
                License: MIT
            conanfile.py (Hello2/0.1)
                URL: myurl
                License: MIT""")

        self.assertIn(expected_output, clean_output(self.client.out))

        self.client.run("info . -u --only=url --only=license --only=description")
        expected_output = textwrap.dedent("""\
            Hello0/0.1@lasote/stable
                URL: myurl
                License: MIT
                Description: blah
            Hello1/0.1@lasote/stable
                URL: myurl
                License: MIT
                Description: blah
            conanfile.py (Hello2/0.1)
                URL: myurl
                License: MIT
                Description: blah""")
        self.assertIn(expected_output, clean_output(self.client.out))

    def test_json_info_outputs(self):
        self.client = TestClient()
        self._create("LibA", "0.1")
        self._create("LibE", "0.1")
        self._create("LibF", "0.1")

        self._create("LibB", "0.1", ["LibA/0.1@lasote/stable", "LibE/0.1@lasote/stable"])
        self._create("LibC", "0.1", ["LibA/0.1@lasote/stable", "LibF/0.1@lasote/stable"])

        self._create("LibD", "0.1", ["LibB/0.1@lasote/stable", "LibC/0.1@lasote/stable"],
                     export=False)

        self.client.run("info . -u --json=output.json")

        # Check a couple of values in the generated JSON
        content = json.loads(self.client.load("output.json"))
        self.assertEqual(content[0]["reference"], "LibA/0.1@lasote/stable")
        self.assertEqual(content[0]["license"][0], "MIT")
        self.assertEqual(content[0]["description"], "blah")
        self.assertEqual(content[0]["revision"], "33574249dee63395e86d2caee3f6c638")
        self.assertEqual(content[0]["package_revision"], None)
        self.assertEqual(content[1]["url"], "myurl")
        self.assertEqual(content[1]["required_by"][0], "conanfile.py (LibD/0.1)")

    def test_build_order(self):
        self.client = TestClient()
        self._create("Hello0", "0.1")
        self._create("Hello1", "0.1", ["Hello0/0.1@lasote/stable"])
        self._create("Hello2", "0.1", ["Hello1/0.1@lasote/stable"], export=False)

        self.client.run("info ./conanfile.py -bo=Hello0/0.1@lasote/stable")
        self.assertIn("[Hello0/0.1@lasote/stable], [Hello1/0.1@lasote/stable]",
                      self.client.out)

        self.client.run("info conanfile.py -bo=Hello1/0.1@lasote/stable")
        self.assertIn("[Hello1/0.1@lasote/stable]", self.client.out)

        self.client.run("info ./ -bo=Hello1/0.1@lasote/stable -bo=Hello0/0.1@lasote/stable")
        self.assertIn("[Hello0/0.1@lasote/stable], [Hello1/0.1@lasote/stable]",
                      self.client.out)

        self.client.run("info Hello1/0.1@lasote/stable -bo=Hello0/0.1@lasote/stable")
        self.assertIn("[Hello0/0.1@lasote/stable], [Hello1/0.1@lasote/stable]\n", self.client.out)

        self.client.run("info Hello1/0.1@lasote/stable -bo=Hello0/0.1@lasote/stable "
                        "--json=file.json")
        self.assertEqual('{"groups": [["Hello0/0.1@lasote/stable"], ["Hello1/0.1@lasote/stable"]]}',
                         self.client.load("file.json"))

        self.client.run("info Hello1/0.1@lasote/stable -bo=Hello0/0.1@lasote/stable --json")
        self.assertIn('{"groups": [["Hello0/0.1@lasote/stable"], ["Hello1/0.1@lasote/stable"]]}',
                      self.client.out)

        self.client.run("info Hello1/0.1@lasote/stable --build-order=Hello0/0.1@lasote/stable "
                        "--graph=index.html", assert_error=True)
        self.assertIn("--build-order cannot be used together with --graph", self.client.out)

    def test_diamond_build_order(self):
        self.client = TestClient()
        self._create("LibA", "0.1")
        self._create("LibE", "0.1")
        self._create("LibF", "0.1")

        self._create("LibB", "0.1", ["LibA/0.1@lasote/stable", "LibE/0.1@lasote/stable"])
        self._create("LibC", "0.1", ["LibA/0.1@lasote/stable", "LibF/0.1@lasote/stable"])

        self._create("LibD", "0.1", ["LibB/0.1@lasote/stable", "LibC/0.1@lasote/stable"],
                     export=False)

        self.client.run("info . -bo=LibA/0.1@lasote/stable")
        self.assertIn("[LibA/0.1@lasote/stable], "
                      "[LibB/0.1@lasote/stable, LibC/0.1@lasote/stable]",
                      self.client.out)
        self.client.run("info . -bo=LibB/0.1@lasote/stable")
        self.assertIn("[LibB/0.1@lasote/stable]", self.client.out)
        self.client.run("info . -bo=LibE/0.1@lasote/stable")
        self.assertIn("[LibE/0.1@lasote/stable], [LibB/0.1@lasote/stable]",
                      self.client.out)
        self.client.run("info . -bo=LibF/0.1@lasote/stable")
        self.assertIn("[LibF/0.1@lasote/stable], [LibC/0.1@lasote/stable]",
                      self.client.out)
        self.client.run("info . -bo=Dev1/0.1@lasote/stable")
        self.assertEqual("WARN: Usage of `--build-order` argument is deprecated and can return wrong"
                         " results. Use `conan lock build-order ...` instead.\n\n", self.client.out)
        self.client.run("info . -bo=LibG/0.1@lasote/stable")
        self.assertEqual("WARN: Usage of `--build-order` argument is deprecated and can return wrong"
                         " results. Use `conan lock build-order ...` instead.\n\n", self.client.out)

        self.client.run("info . --build-order=ALL")
        self.assertIn("[LibA/0.1@lasote/stable, LibE/0.1@lasote/stable, LibF/0.1@lasote/stable], "
                      "[LibB/0.1@lasote/stable, LibC/0.1@lasote/stable]",
                      self.client.out)

        self.client.run("info . --build-order=ALL")
        self.assertIn("[LibA/0.1@lasote/stable, LibE/0.1@lasote/stable, "
                      "LibF/0.1@lasote/stable], [LibB/0.1@lasote/stable, LibC/0.1@lasote/stable]",
                      self.client.out)


class InfoTest2(unittest.TestCase):

    def test_not_found_package_dirty_cache(self):
        # Conan does a lock on the cache, and even if the package doesn't exist
        # left a trailing folder with the filelocks. This test checks
        # it will be cleared
        client = TestClient()
        client.run("info nothing/0.1@user/testing", assert_error=True)
        self.assertEqual(os.listdir(client.cache.store), [])
        # This used to fail in Windows, because of the different case
        client.save({"conanfile.py": GenConanfile().with_name("Nothing").with_version("0.1")})
        client.run("export . user/testing")

    def test_failed_info(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile().with_require("Pkg/1.0.x@user/testing")})
        client.run("info .", assert_error=True)
        self.assertIn("Pkg/1.0.x@user/testing: Not found in local cache", client.out)
        client.run("search")
        self.assertIn("There are no packages", client.out)
        self.assertNotIn("Pkg/1.0.x@user/testing", client.out)

    def test_install_folder(self):
        conanfile = GenConanfile("Pkg", "0.1").with_setting("build_type")
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("info . -s build_type=Debug")
        self.assertNotIn("ID: 4024617540c4f240a6a5e8911b0de9ef38a11a72", client.out)
        self.assertIn("ID: 5a67a79dbc25fd0fa149a0eb7a20715189a0d988", client.out)

        client.run('install . -s build_type=Debug')
        client.run("info .")  # Re-uses debug from curdir
        self.assertNotIn("ID: 4024617540c4f240a6a5e8911b0de9ef38a11a72", client.out)
        self.assertIn("ID: 5a67a79dbc25fd0fa149a0eb7a20715189a0d988", client.out)

        client.run('install . -s build_type=Release --install-folder=MyInstall')
        client.run("info . --install-folder=MyInstall")  # Re-uses debug from MyInstall folder

        self.assertIn("ID: 4024617540c4f240a6a5e8911b0de9ef38a11a72", client.out)
        self.assertNotIn("ID: 5a67a79dbc25fd0fa149a0eb7a20715189a0d988", client.out)

        client.run('install . -s build_type=Debug --install-folder=MyInstall')
        client.run("info . --install-folder=MyInstall")  # Re-uses debug from MyInstall folder

        self.assertNotIn("ID: 4024617540c4f240a6a5e8911b0de9ef38a11a72", client.out)
        self.assertIn("ID: 5a67a79dbc25fd0fa149a0eb7a20715189a0d988", client.out)

        # Both should raise
        client.run("info . --install-folder=MyInstall -s build_type=Release",
                   assert_error=True)  # Re-uses debug from MyInstall folder

        self.assertIn("--install-folder cannot be used together with a"
                      " host profile (-s, -o, -e, -pr or -c)", client.out)

    def test_graph_html_embedded_visj(self):
        client = TestClient()
        visjs_path = os.path.join(client.cache_folder, "vis.min.js")
        viscss_path = os.path.join(client.cache_folder, "vis.min.css")
        save(visjs_path, "")
        save(viscss_path, "")
        client.save({"conanfile.txt": ""})
        client.run("info . --graph=file.html")
        html = client.load("file.html")
        self.assertIn("<body>", html)
        self.assertNotIn("cloudflare", html)
        self.assertIn(visjs_path, html)
        self.assertIn(viscss_path, html)

    def test_info_build_requires(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . tool/0.1@user/channel")
        client.run("create . dep/0.1@user/channel")
        conanfile = GenConanfile().with_require("dep/0.1@user/channel")
        client.save({"conanfile.py": conanfile})
        client.run("export . Pkg/0.1@user/channel")
        client.run("export . Pkg2/0.1@user/channel")
        client.save({"conanfile.txt": "[requires]\nPkg/0.1@user/channel\nPkg2/0.1@user/channel",
                     "myprofile": "[build_requires]\ntool/0.1@user/channel"}, clean_first=True)
        client.run("info . -pr=myprofile --dry-build=missing")
        # Check that there is only 1 output for tool, not repeated many times
        pkgs = [line for line in str(client.out).splitlines() if line.startswith("tool")]
        self.assertEqual(len(pkgs), 1)

        client.run("info . -pr=myprofile --dry-build=missing --graph=file.html")
        html = client.load("file.html")
        self.assertIn("html", html)
        # To check that this node is not duplicated
        self.assertEqual(1, html.count("label: 'dep/0.1'"))
        self.assertIn("label: 'Pkg2/0.1',\n                        "
                      "shape: 'box',\n                        "
                      "color: { background: 'Khaki'},", html)
        self.assertIn("label: 'Pkg/0.1',\n                        "
                      "shape: 'box',\n                        "
                      "color: { background: 'Khaki'},", html)
        self.assertIn("label: 'tool/0.1',\n                        "
                      "shape: 'ellipse',\n                        "
                      "color: { background: 'SkyBlue'},", html)

    def test_cwd(self):
        client = TestClient()
        conanfile = GenConanfile("Pkg", "0.1").with_setting("build_type")
        client.save({"subfolder/conanfile.py": conanfile})
        client.run("export ./subfolder lasote/testing")

        client.run("info ./subfolder")
        self.assertIn("conanfile.py (Pkg/0.1)", client.out)

        client.run("info ./subfolder --build-order Pkg/0.1@lasote/testing --json=jsonfile.txt")
        path = os.path.join(client.current_folder, "jsonfile.txt")
        self.assertTrue(os.path.exists(path))

    def test_build_order_build_requires(self):
        # https://github.com/conan-io/conan/issues/3267
        client = TestClient()
        conanfile = str(GenConanfile())
        client.save({"conanfile.py": conanfile})
        client.run("create . tool/0.1@user/channel")
        client.run("create . dep/0.1@user/channel")
        conanfile = conanfile + 'requires = "dep/0.1@user/channel"'
        client.save({"conanfile.py": conanfile})
        client.run("export . Pkg/0.1@user/channel")
        client.run("export . Pkg2/0.1@user/channel")
        client.save({"conanfile.txt": "[requires]\nPkg/0.1@user/channel\nPkg2/0.1@user/channel",
                     "myprofile": "[build_requires]\ntool/0.1@user/channel"}, clean_first=True)
        client.run("info . -pr=myprofile -bo=tool/0.1@user/channel")
        self.assertIn("[tool/0.1@user/channel], [Pkg/0.1@user/channel, Pkg2/0.1@user/channel]",
                      client.out)

    def test_build_order_privates(self):
        # https://github.com/conan-io/conan/issues/3267
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . tool/0.1@user/channel")
        client.save({"conanfile.py": GenConanfile().with_require("tool/0.1@user/channel")})
        client.run("create . dep/0.1@user/channel")
        client.save({"conanfile.py": GenConanfile().with_require("dep/0.1@user/channel",
                                                                 private=True)})
        client.run("export . Pkg/0.1@user/channel")
        client.run("export . Pkg2/0.1@user/channel")
        client.save({"conanfile.txt": "[requires]\nPkg/0.1@user/channel\nPkg2/0.1@user/channel"},
                    clean_first=True)
        client.run("info . -bo=tool/0.1@user/channel")
        self.assertIn("[tool/0.1@user/channel], [dep/0.1@user/channel], "
                      "[Pkg/0.1@user/channel, Pkg2/0.1@user/channel]",
                      client.out)

    def test_wrong_path_parameter(self):
        client = TestClient()

        client.run("info", assert_error=True)
        self.assertIn("ERROR: Exiting with code: 2", client.out)

        client.run("info not_real_path", assert_error=True)
        self.assertIn("ERROR: Conanfile not found", client.out)

        client.run("info conanfile.txt", assert_error=True)
        self.assertIn("ERROR: Conanfile not found", client.out)

    def test_common_attributes(self):
        client = TestClient()

        conanfile = GenConanfile("Pkg", "0.1").with_setting("build_type")
        client.save({"subfolder/conanfile.py": conanfile})
        client.run("export ./subfolder lasote/testing")

        client.run("info ./subfolder")

        self.assertIn("conanfile.py (Pkg/0.1)", client.out)
        self.assertNotIn("License:", client.out)
        self.assertNotIn("Author:", client.out)
        self.assertNotIn("Topics:", client.out)
        self.assertNotIn("Homepage:", client.out)
        self.assertNotIn("URL:", client.out)

    def test_full_attributes(self):
        client = TestClient()

        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class MyTest(ConanFile):
                name = "Pkg"
                version = "0.2"
                settings = "build_type"
                author = "John Doe"
                license = "MIT"
                url = "https://foo.bar.baz"
                homepage = "https://foo.bar.site"
                topics = ("foo", "bar", "qux")
                provides = ("libjpeg", "libjpg")
                deprecated = "other-pkg"
        """)

        client.save({"subfolder/conanfile.py": conanfile})
        client.run("export ./subfolder lasote/testing")
        client.run("info ./subfolder")

        self.assertIn("conanfile.py (Pkg/0.2)", client.out)
        self.assertIn("License: MIT", client.out)
        self.assertIn("Author: John Doe", client.out)
        self.assertIn("Topics: foo, bar, qux", client.out)
        self.assertIn("URL: https://foo.bar.baz", client.out)
        self.assertIn("Homepage: https://foo.bar.site", client.out)
        self.assertIn("Provides: libjpeg, libjpg", client.out)
        self.assertIn("Deprecated: other-pkg", client.out)

        client.run("info ./subfolder --json=output.json")
        output = json.loads(client.load('output.json'))[0]
        self.assertEqual(output['reference'], 'conanfile.py (Pkg/0.2)')
        self.assertListEqual(output['license'], ['MIT', ])
        self.assertEqual(output['author'], 'John Doe')
        self.assertListEqual(output['topics'], ['foo', 'bar', 'qux'])
        self.assertEqual(output['url'], 'https://foo.bar.baz')
        self.assertEqual(output['homepage'], 'https://foo.bar.site')
        self.assertListEqual(output['provides'], ['libjpeg', 'libjpg'])
        self.assertEqual(output['deprecated'], 'other-pkg')

    def test_topics_graph(self):

        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class MyTest(ConanFile):
                name = "Pkg"
                version = "0.2"
                topics = ("foo", "bar", "qux")
        """)

        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("export . lasote/testing")

        # Topics as tuple
        client.run("info Pkg/0.2@lasote/testing --graph file.html")
        html_content = client.load("file.html")
        self.assertIn("<h3>Pkg/0.2@lasote/testing</h3>", html_content)
        self.assertIn("<li><b>topics</b>: foo, bar, qux</li>", html_content)

        # Topics as a string
        conanfile = conanfile.replace("(\"foo\", \"bar\", \"qux\")", "\"foo\"")
        client.save({"conanfile.py": conanfile}, clean_first=True)
        client.run("export . lasote/testing")
        client.run("info Pkg/0.2@lasote/testing --graph file.html")
        html_content = client.load("file.html")
        self.assertIn("<h3>Pkg/0.2@lasote/testing</h3>", html_content)
        self.assertIn("<li><b>topics</b>: foo", html_content)

    def test_wrong_graph_info(self):
        # https://github.com/conan-io/conan/issues/4443
        conanfile = GenConanfile().with_name("Hello").with_version("0.1")
        client = TestClient()
        client.save({"conanfile.py": str(conanfile)})
        client.run("install .")
        path = os.path.join(client.current_folder, "graph_info.json")
        graph_info = client.load(path)
        graph_info = json.loads(graph_info)
        graph_info.pop("root")
        save(path, json.dumps(graph_info))
        client.run("info .")
        self.assertIn("conanfile.py (Hello/0.1)", client.out)
        save(path, "broken thing")
        client.run("info .", assert_error=True)
        self.assertIn("ERROR: Error parsing GraphInfo from file", client.out)

    def test_previous_lockfile_error(self):
        # https://github.com/conan-io/conan/issues/5479
        client = TestClient()
        client.save({"conanfile.py": GenConanfile().with_name("pkg").with_version("0.1")})
        client.run("create . user/testing")
        client.save({"conanfile.py": GenConanfile().with_name("other").with_version("0.1")
                    .with_option("shared", [True, False])
                    .with_default_option("shared", False)})
        client.run("install . -o shared=True")
        client.run("info pkg/0.1@user/testing")
        self.assertIn("pkg/0.1@user/testing", client.out)
        self.assertNotIn("shared", client.out)


def test_scm_info():
    # https://github.com/conan-io/conan/issues/8377
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class Pkg(ConanFile):
            scm = {"type": "git",
                   "url": "some-url/path",
                   "revision": "some commit hash"}
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("export . pkg/0.1@")

    client.run("info .")
    assert "'revision': 'some commit hash'" in client.out
    assert "'url': 'some-url/path'" in client.out

    client.run("info pkg/0.1@")
    assert "'revision': 'some commit hash'" in client.out
    assert "'url': 'some-url/path'" in client.out

    client.run("info . --json=file.json")
    file_json = client.load("file.json")
    info_json = json.loads(file_json)
    node = info_json[0]
    assert node["scm"] == {"type": "git", "url": "some-url/path", "revision": "some commit hash"}


class TestInfoContext:
    # https://github.com/conan-io/conan/issues/9121

    def test_context_info(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})

        client.run("info .")
        assert "Context: host" in client.out

        client.run("info . --json=file.json")
        info = json.loads(client.load("file.json"))
        assert info[0]["context"] == "host"

    def test_context_build(self):
        client = TestClient()
        client.save({"cmake/conanfile.py": GenConanfile(),
                     "pkg/conanfile.py": GenConanfile().with_build_requires("cmake/1.0")})

        client.run("create cmake cmake/1.0@")
        client.run("export pkg pkg/1.0@")

        client.run("info pkg/1.0@ -pr:b=default -pr:h=default --dry-build")
        assert "cmake/1.0\n"\
               "    ID: {}\n"\
               "    BuildID: None\n"\
               "    Context: build".format(NO_SETTINGS_PACKAGE_ID) in client.out

        assert "pkg/1.0\n" \
               "    ID: {}\n" \
               "    BuildID: None\n" \
               "    Context: host".format(NO_SETTINGS_PACKAGE_ID) in client.out

        client.run("info pkg/1.0@ -pr:b=default -pr:h=default --dry-build --json=file.json")
        info = json.loads(client.load("file.json"))
        assert info[0]["reference"] == "cmake/1.0"
        assert info[0]["context"] == "build"
        assert info[1]["reference"] == "pkg/1.0"
        assert info[1]["context"] == "host"


class TestInfoPythonRequires:
    # https://github.com/conan-io/conan/issues/9277

    def test_python_requires(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("export . tool/0.1@")
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                python_requires = "tool/0.1"
            """)
        client.save({"conanfile.py": conanfile})

        client.run("info .")
        assert "Python-requires:" in client.out
        assert "tool/0.1#f3367e0e7d170aa12abccb175fee5f97" in client.out

        client.run("info . --json=file.json")
        info = json.loads(client.load("file.json"))
        assert info[0]["python_requires"] == ['tool/0.1#f3367e0e7d170aa12abccb175fee5f97']

class TestInfoTestPackage:
    # https://github.com/conan-io/conan/issues/10714

    def test_tested_reference_str(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("export . tool/0.1@")

        conanfile = textwrap.dedent("""
from conans import ConanFile
class HelloConan(ConanFile):

    def requirements(self):
        self.requires(self.tested_reference_str)

    def build_requirements(self):
        self.build_requires(self.tested_reference_str)

    test_type = 'explicit'
""")
        client.save({"conanfile.py": conanfile})

        for args in ["", " --build-order tool/0.1@", " --build"]:
            client.run("info . " + args)
            assert "AttributeError: 'HelloConan' object has no attribute 'tested_reference_str'"\
                   not in client.out



import os
import textwrap
import unittest

from requests.models import Response

from conans.client import tools
from conans.errors import AuthenticationException
from conans.model.ref import ConanFileReference
from conans.paths import CONANFILE
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.test_files import temp_folder
from conans.test.utils.tools import TestClient
from conans.test.utils.tools import TestServer
from conans.util.files import save

conan_content = """
from conans import ConanFile

class OpenSSLConan(ConanFile):
    name = "openssl"
    version = "2.0.1"
    files = '*'
"""


class AuthorizeTest(unittest.TestCase):

    def setUp(self):
        self.servers = {}
        self.ref = ConanFileReference.loads("openssl/2.0.1@lasote/testing")
        # Create a default remote. R/W is not authorized for ref,
        # just for pepe, nacho and owner
        self.test_server = TestServer([(str(self.ref), "pepe,nacho@gmail.com")],  # read permissions
                                      [(str(self.ref), "pepe,nacho@gmail.com")],  # write permissions
                                      users={"lasote": "mypass",
                                             "pepe": "pepepass",
                                             "nacho@gmail.com": "nachopass"})  # exported creds
        self.servers["default"] = self.test_server

    def test_retries(self):
        """Bad login 2 times"""
        self.conan = TestClient(servers=self.servers, users={"default": [("baduser", "badpass"),
                                                                         ("baduser", "badpass2"),
                                                                         ("pepe", "pepepass")]})
        save(os.path.join(self.conan.current_folder, CONANFILE), conan_content)
        self.conan.run("export . lasote/testing")
        errors = self.conan.run("upload %s" % str(self.ref))
        # Check that return was  ok
        self.assertFalse(errors)
        # Check that upload was granted
        rev = self.test_server.server_store.get_last_revision(self.ref).revision
        ref = self.ref.copy_with_rev(rev)
        self.assertTrue(os.path.exists(self.test_server.server_store.export(ref)))

        # Check that login failed two times before ok
        self.assertEqual(self.conan.api.app.user_io.login_index["default"], 3)

    def test_auth_with_env(self):

        def _upload_with_credentials(credentials):
            cli = TestClient(servers=self.servers, users={})
            save(os.path.join(cli.current_folder, CONANFILE), conan_content)
            cli.run("export . lasote/testing")
            with tools.environment_append(credentials):
                cli.run("upload %s" % str(self.ref))
            return cli

        # Try with remote name in credentials
        client = _upload_with_credentials({"CONAN_PASSWORD_DEFAULT": "pepepass",
                                           "CONAN_LOGIN_USERNAME_DEFAULT": "pepe"})
        self.assertIn("Got username 'pepe' from environment", client.out)
        self.assertIn("Got password '******' from environment", client.out)

        # Try with generic password and login
        client = _upload_with_credentials({"CONAN_PASSWORD": "pepepass",
                                           "CONAN_LOGIN_USERNAME_DEFAULT": "pepe"})
        self.assertIn("Got username 'pepe' from environment", client.out)
        self.assertIn("Got password '******' from environment", client.out)

        # Try with generic password and generic login
        client = _upload_with_credentials({"CONAN_PASSWORD": "pepepass",
                                           "CONAN_LOGIN_USERNAME": "pepe"})
        self.assertIn("Got username 'pepe' from environment", client.out)
        self.assertIn("Got password '******' from environment", client.out)

        # Bad pass raise
        with self.assertRaises(Exception):
            client = _upload_with_credentials({"CONAN_PASSWORD": "bad",
                                               "CONAN_LOGIN_USERNAME": "pepe"})
            self.assertIn("Too many failed login attempts, bye!", client.out)

    def test_max_retries(self):
        """Bad login 3 times"""
        self.conan = TestClient(servers=self.servers, users={"default": [("baduser", "badpass"),
                                                                         ("baduser", "badpass2"),
                                                                         ("baduser3", "badpass3")]})
        save(os.path.join(self.conan.current_folder, CONANFILE), conan_content)
        self.conan.run("export . lasote/testing")
        errors = self.conan.run("upload %s" % str(self.ref), assert_error=True)
        # Check that return was not ok
        self.assertTrue(errors)
        # Check that upload was not granted
        rev = self.servers["default"].server_store.get_last_revision(self.ref)
        self.assertIsNone(rev)

        # Check that login failed all times
        self.assertEqual(self.conan.api.app.user_io.login_index["default"], 3)

    def test_no_client_username_checks(self):
        """Checks whether client username checks are disabled."""

        # Try with a load of names that contain special characters
        self.conan = TestClient(servers=self.servers, users={"default": [
                                        ("some_random.special!characters", "badpass"),
                                        ("nacho@gmail.com", "nachopass"),
                                        ]})
        save(os.path.join(self.conan.current_folder, CONANFILE), conan_content)
        self.conan.run("export . lasote/testing")
        self.conan.run("upload %s" % str(self.ref))

        # Check that upload was granted
        rev = self.test_server.server_store.get_last_revision(self.ref).revision
        ref = self.ref.copy_with_rev(rev)
        self.assertTrue(os.path.exists(self.test_server.server_store.export(ref)))

        # Check that login failed once before ok
        self.assertEqual(self.conan.api.app.user_io.login_index["default"], 2)


class AuthenticationTest(unittest.TestCase):

    def test_unauthorized_during_capabilities(self):

        class RequesterMock(object):

            def __init__(self, *args, **kwargs):
                pass

            @staticmethod
            def get(url, **kwargs):
                resp_basic_auth = Response()
                resp_basic_auth.status_code = 200
                if "authenticate" in url:
                    if kwargs["auth"].password != "PASSWORD!":
                        raise Exception("Bad password")
                    resp_basic_auth._content = b"TOKEN"
                    resp_basic_auth.headers = {"Content-Type": "text/plain"}
                elif "ping" in url:
                    token = getattr(kwargs["auth"], "token", None)
                    password = getattr(kwargs["auth"], "password", None)
                    if token and token != "TOKEN":
                        raise Exception("Bad JWT Token")
                    if not token and not password:
                        raise AuthenticationException(
                            "I'm an Artifactory without anonymous access that "
                            "requires authentication for the ping endpoint and "
                            "I don't return the capabilities")
                elif "search" in url:
                    if kwargs["auth"].token != "TOKEN":
                        raise Exception("Bad JWT Token")
                    resp_basic_auth._content = b'{"results": []}'
                    resp_basic_auth.headers = {"Content-Type": "application/json"}
                else:
                    raise Exception("Shouldn't be more remote calls")
                return resp_basic_auth

        client = TestClient(requester_class=RequesterMock, default_server_user=True)
        client.run("user user -p PASSWORD! -r=default")
        self.assertIn("Changed user of remote 'default' from 'None' (anonymous) to 'user'",
                      client.out)
        client.run("search pkg -r=default")
        self.assertIn("There are no packages matching the 'pkg' pattern", client.out)


def test_token_expired():
    server_folder = temp_folder()
    server_conf = textwrap.dedent("""
       [server]
       jwt_expire_minutes: 0.01
       authorize_timeout: 0
       disk_authorize_timeout: 0
       disk_storage_path: ./data
       updown_secret: 12345
       jwt_secret: mysecret
       port: 12345
       [read_permissions]
       */*@*/*: *
       [write_permissions]
       */*@*/*: admin
       """)
    save(os.path.join(server_folder, ".conan_server", "server.conf"), server_conf)
    server = TestServer(base_path=server_folder, users={"admin": "password"})

    c = TestClient(servers={"default": server}, users={"default": [("admin", "password")]})
    c.save({"conanfile.py": GenConanfile()})
    c.run("create . pkg/0.1@user/stable")
    c.run("upload * -r=default --all -c")
    user, token, _ = c.cache.localdb.get_login(server.fake_url)
    assert user == "admin"
    assert token is not None

    import time
    time.sleep(2)
    c.users = {}
    c.run("config set general.non_interactive=1")
    c.run("remove * -f")
    c.run("install pkg/0.1@user/stable")
    user, token, _ = c.cache.localdb.get_login(server.fake_url)
    assert user == "admin"
    assert token is None



import textwrap
import unittest

from conans.test.utils.tools import TestClient


class BuildHelperTest(unittest.TestCase):
    def test_autotools_helper(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile, AutoToolsBuildEnvironment

            class Pkg(ConanFile):
                def build(self):
                    AutoToolsBuildEnvironment(self)

        """)
        client.save({"conanfile.py": conanfile,
                     "host": "",
                     "build": ""})
        client.run("create . pkg/1.0@ --profile:build=build --profile:host=host")
        self.assertIn("Configuration (profile_host):", client.out)
        self.assertIn("Configuration (profile_build):", client.out)
        self.assertIn("pkg/1.0: Calling build()", client.out)
        self.assertIn("pkg/1.0: Created package", client.out)



import unittest

from mock import Mock

from conans.client.conf import get_default_settings_yml
from conans.client.generators.pkg_config import PkgConfigGenerator
from conans.model.build_info import CppInfo
from conans.model.conan_file import ConanFile
from conans.model.env_info import EnvValues
from conans.model.ref import ConanFileReference
from conans.model.settings import Settings


class PkgGeneratorTest(unittest.TestCase):

    def test_variables_setup(self):
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(Settings({}), EnvValues())
        ref = ConanFileReference.loads("MyPkg/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "/dummy_root_folder1")
        cpp_info.filter_empty = False
        cpp_info.name = "my_pkg"
        cpp_info.defines = ["MYDEFINE1"]
        cpp_info.cflags.append("-Flag1=23")
        cpp_info.version = "1.3"
        cpp_info.description = "My cool description"
        conanfile.deps_cpp_info.add(ref.name, cpp_info)

        ref = ConanFileReference.loads("MyPkg1/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "/dummy_root_folder1")
        cpp_info.filter_empty = False
        cpp_info.name = "MYPKG1"
        cpp_info.defines = ["MYDEFINE11"]
        cpp_info.cflags.append("-Flag1=21")
        cpp_info.version = "1.7"
        cpp_info.description = "My other cool description"
        cpp_info.public_deps = ["MyPkg"]
        conanfile.deps_cpp_info.add(ref.name, cpp_info)

        ref = ConanFileReference.loads("MyPkg2/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "/dummy_root_folder2")
        cpp_info.filter_empty = False
        cpp_info.defines = ["MYDEFINE2"]
        cpp_info.version = "2.3"
        cpp_info.exelinkflags = ["-exelinkflag"]
        cpp_info.sharedlinkflags = ["-sharedlinkflag"]
        cpp_info.cxxflags = ["-cxxflag"]
        cpp_info.public_deps = ["MyPkg"]
        conanfile.deps_cpp_info.add(ref.name, cpp_info)
        generator = PkgConfigGenerator(conanfile)
        files = generator.content

        self.assertEqual(files["MyPkg2.pc"], """prefix=/dummy_root_folder2
libdir=${prefix}/lib
includedir=${prefix}/include

Name: MyPkg2
Description: Conan package: MyPkg2
Version: 2.3
Libs: -L"${libdir}" -sharedlinkflag -exelinkflag
Cflags: -I"${includedir}" -cxxflag -DMYDEFINE2
Requires: my_pkg
""")

        self.assertEqual(files["mypkg1.pc"], """prefix=/dummy_root_folder1
libdir=${prefix}/lib
includedir=${prefix}/include

Name: mypkg1
Description: My other cool description
Version: 1.7
Libs: -L"${libdir}"
Cflags: -I"${includedir}" -Flag1=21 -DMYDEFINE11
Requires: my_pkg
""")

        self.assertEqual(files["my_pkg.pc"], """prefix=/dummy_root_folder1
libdir=${prefix}/lib
includedir=${prefix}/include

Name: my_pkg
Description: My cool description
Version: 1.3
Libs: -L"${libdir}"
Cflags: -I"${includedir}" -Flag1=23 -DMYDEFINE1
""")

    def test_pkg_config_custom_names(self):
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(Settings({}), EnvValues())

        ref = ConanFileReference.loads("MyPkg/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "/dummy_root_folder1")
        cpp_info.filter_empty = False
        cpp_info.name = "my_pkg"
        cpp_info.names["pkg_config"] = "my_pkg_custom_name"
        cpp_info.defines = ["MYDEFINE1"]
        cpp_info.cflags.append("-Flag1=23")
        cpp_info.version = "1.3"
        cpp_info.description = "My cool description"
        conanfile.deps_cpp_info.add(ref.name, cpp_info)

        ref = ConanFileReference.loads("MyPkg1/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "/dummy_root_folder1")
        cpp_info.filter_empty = False
        cpp_info.name = "MYPKG1"
        cpp_info.names["pkg_config"] = "my_pkg1_custom_name"
        cpp_info.defines = ["MYDEFINE11"]
        cpp_info.cflags.append("-Flag1=21")
        cpp_info.version = "1.7"
        cpp_info.description = "My other cool description"
        conanfile.deps_cpp_info.add(ref.name, cpp_info)

        ref = ConanFileReference.loads("MyPkg2/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "/dummy_root_folder2")
        cpp_info.filter_empty = False
        cpp_info.names["pkg_config"] = "my_pkg2_custom_name"
        cpp_info.defines = ["MYDEFINE2"]
        cpp_info.version = "2.3"
        cpp_info.exelinkflags = ["-exelinkflag"]
        cpp_info.sharedlinkflags = ["-sharedlinkflag"]
        cpp_info.cxxflags = ["-cxxflag"]
        cpp_info.public_deps = ["MyPkg", "MyPkg1"]
        conanfile.deps_cpp_info.add(ref.name, cpp_info)

        ref = ConanFileReference.loads("zlib/1.2.11@lasote/stable")
        cpp_info = CppInfo(ref.name, "/dummy_root_folder_zlib")
        cpp_info.filter_empty = False
        cpp_info.name = "ZLIB"
        cpp_info.defines = ["MYZLIBDEFINE2"]
        cpp_info.version = "1.2.11"
        conanfile.deps_cpp_info.add(ref.name, cpp_info)

        ref = ConanFileReference.loads("bzip2/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "/dummy_root_folder2")
        cpp_info.filter_empty = False
        cpp_info.name = "BZip2"
        cpp_info.names["pkg_config"] = "BZip2"
        cpp_info.defines = ["MYDEFINE2"]
        cpp_info.version = "2.3"
        cpp_info.exelinkflags = ["-exelinkflag"]
        cpp_info.sharedlinkflags = ["-sharedlinkflag"]
        cpp_info.cxxflags = ["-cxxflag"]
        cpp_info.public_deps = ["MyPkg", "MyPkg1", "zlib"]
        conanfile.deps_cpp_info.add(ref.name, cpp_info)
        generator = PkgConfigGenerator(conanfile)
        files = generator.content

        self.assertEqual(files["my_pkg2_custom_name.pc"], """prefix=/dummy_root_folder2
libdir=${prefix}/lib
includedir=${prefix}/include

Name: my_pkg2_custom_name
Description: Conan package: my_pkg2_custom_name
Version: 2.3
Libs: -L"${libdir}" -sharedlinkflag -exelinkflag
Cflags: -I"${includedir}" -cxxflag -DMYDEFINE2
Requires: my_pkg_custom_name my_pkg1_custom_name
""")
        self.assertEqual(files["my_pkg1_custom_name.pc"], """prefix=/dummy_root_folder1
libdir=${prefix}/lib
includedir=${prefix}/include

Name: my_pkg1_custom_name
Description: My other cool description
Version: 1.7
Libs: -L"${libdir}"
Cflags: -I"${includedir}" -Flag1=21 -DMYDEFINE11
""")
        self.assertEqual(files["my_pkg_custom_name.pc"], """prefix=/dummy_root_folder1
libdir=${prefix}/lib
includedir=${prefix}/include

Name: my_pkg_custom_name
Description: My cool description
Version: 1.3
Libs: -L"${libdir}"
Cflags: -I"${includedir}" -Flag1=23 -DMYDEFINE1
""")
        self.assertEqual(files["BZip2.pc"], """prefix=/dummy_root_folder2
libdir=${prefix}/lib
includedir=${prefix}/include

Name: BZip2
Description: Conan package: BZip2
Version: 2.3
Libs: -L"${libdir}" -sharedlinkflag -exelinkflag
Cflags: -I"${includedir}" -cxxflag -DMYDEFINE2
Requires: my_pkg_custom_name my_pkg1_custom_name zlib
""")

    def test_apple_frameworks(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.compiler = "apple-clang"
        settings.os = "Macos"
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(Settings({}), EnvValues())
        conanfile.settings = settings
        ref = ConanFileReference.loads("MyPkg/0.1@lasote/stables")
        cpp_info = CppInfo(ref.name, "/dummy_root_folder1")
        cpp_info.filter_empty = False
        cpp_info.frameworks = ['AudioUnit', 'AudioToolbox']
        cpp_info.version = "1.3"
        cpp_info.description = "My cool description"
        conanfile.deps_cpp_info.add(ref.name, cpp_info)

        generator = PkgConfigGenerator(conanfile)
        files = generator.content

        self.assertEqual(files["MyPkg.pc"], """prefix=/dummy_root_folder1
libdir=${prefix}/lib
includedir=${prefix}/include

Name: MyPkg
Description: My cool description
Version: 1.3
Libs: -L"${libdir}" -Wl,-rpath,"${libdir}" -framework AudioUnit -framework AudioToolbox -F /dummy_root_folder1/Frameworks
Cflags: -I"${includedir}"
""")



import unittest

from conans.client import tools


class AndroidTest(unittest.TestCase):
    def test_to_android_abi(self):
        self.assertEqual(tools.to_android_abi('x86'), 'x86')
        self.assertEqual(tools.to_android_abi('x86_64'), 'x86_64')
        self.assertEqual(tools.to_android_abi('armv5'), 'armeabi')
        self.assertEqual(tools.to_android_abi('armv5el'), 'armeabi')
        self.assertEqual(tools.to_android_abi('armv5hf'), 'armeabi')
        self.assertEqual(tools.to_android_abi('armv6'), 'armeabi-v6')
        self.assertEqual(tools.to_android_abi('armv7'), 'armeabi-v7a')
        self.assertEqual(tools.to_android_abi('armv7hf'), 'armeabi-v7a')
        self.assertEqual(tools.to_android_abi('armv8'), 'arm64-v8a')
        self.assertEqual(tools.to_android_abi('mips'), 'mips')
        self.assertEqual(tools.to_android_abi('mips64'), 'mips64')



import platform
import textwrap

import pytest

from conans.test.assets.sources import gen_function_cpp
from conans.test.conftest import tools_locations
from conans.test.utils.tools import TestClient


class TestToolsCustomVersions:

    @pytest.mark.tool_cmake
    def test_default_cmake(self):
        client = TestClient()
        client.run_command('cmake --version')
        default_cmake_version = tools_locations["cmake"]["default"]
        assert "cmake version {}".format(default_cmake_version) in client.out

    @pytest.mark.tool_cmake(version="3.16")
    def test_custom_cmake_3_16(self):
        client = TestClient()
        client.run_command('cmake --version')
        assert "cmake version 3.16" in client.out

    @pytest.mark.tool_cmake(version="3.17")
    def test_custom_cmake_3_17(self):
        client = TestClient()
        client.run_command('cmake --version')
        assert "cmake version 3.17" in client.out

    @pytest.mark.tool_cmake(version="3.19")
    def test_custom_cmake_3_19(self):
        client = TestClient()
        client.run_command('cmake --version')
        assert "cmake version 3.19" in client.out

    @pytest.mark.tool_mingw64
    @pytest.mark.tool_cmake(version="3.16")
    @pytest.mark.skipif(platform.system() != "Windows",
                        reason="Mingw test")
    def test_custom_cmake_mingw64(self):
        client = TestClient()
        client.run_command('cmake --version')
        assert "cmake version 3.16" in client.out
        main = gen_function_cpp(name="main")
        cmakelist = textwrap.dedent("""
            cmake_minimum_required(VERSION 3.15)
            project(App C CXX)
            add_executable(app app.cpp)
            """)
        client.save({"CMakeLists.txt": cmakelist, "app.cpp": main})
        client.run_command('cmake . -G "MinGW Makefiles"')
        client.run_command("cmake --build .")



import os
import textwrap

import pytest
from conans import load

from conan.tools import CONAN_TOOLCHAIN_ARGS_FILE, CONAN_TOOLCHAIN_ARGS_SECTION
from conan.tools.files.files import load_toolchain_args, save_toolchain_args
from conans.errors import ConanException
from conans.test.utils.test_files import temp_folder
from conans.util.files import save, remove


def test_load_empty_toolchain_args_in_default_dir():
    folder = temp_folder()
    os.chdir(folder)
    save(CONAN_TOOLCHAIN_ARGS_FILE, "[%s]" % CONAN_TOOLCHAIN_ARGS_SECTION)
    try:
        config = load_toolchain_args()
        assert not config
    finally:
        remove(CONAN_TOOLCHAIN_ARGS_FILE)


def test_load_toolchain_args_if_it_does_not_exist():
    folder = temp_folder()
    os.chdir(folder)
    with pytest.raises(ConanException):
        load_toolchain_args()


def test_toolchain_args_with_content_full():
    folder = temp_folder()
    content = textwrap.dedent(r"""
    [%s]
    win_path=my\win\path
    command=conan --option "My Option"
    my_regex=([A-Z])\w+
    """ % CONAN_TOOLCHAIN_ARGS_SECTION)
    save(os.path.join(folder,  CONAN_TOOLCHAIN_ARGS_FILE), content)
    args = load_toolchain_args(generators_folder=folder)
    assert args["win_path"] == r'my\win\path'
    assert args["command"] == r'conan --option "My Option"'
    assert args["my_regex"] == r'([A-Z])\w+'


def test_save_toolchain_args_empty():
    folder = temp_folder()
    content = {}
    save_toolchain_args(content, generators_folder=folder)
    args = load(os.path.join(folder, CONAN_TOOLCHAIN_ARGS_FILE))
    assert "[%s]" % CONAN_TOOLCHAIN_ARGS_SECTION in args


def test_save_toolchain_args_full():
    folder = temp_folder()
    content = {
        'win_path': r'my\win\path',
        'command': r'conan --option "My Option"',
        'my_regex': r'([A-Z])\w+'
    }
    save_toolchain_args(content, generators_folder=folder)
    args = load(os.path.join(folder, CONAN_TOOLCHAIN_ARGS_FILE))
    assert "[%s]" % CONAN_TOOLCHAIN_ARGS_SECTION in args
    assert r'win_path = my\win\path' in args



import os
import textwrap
import unittest
from collections import namedtuple

from mock import Mock

from conans.client.cache.cache import ClientCache
from conans.client.cache.remote_registry import Remotes
from conans.client.generators import GeneratorManager
from conans.client.graph.build_mode import BuildMode
from conans.client.graph.graph_binaries import GraphBinariesAnalyzer
from conans.client.graph.graph_manager import GraphManager
from conans.client.graph.proxy import ConanProxy
from conans.client.graph.python_requires import ConanPythonRequire, PyRequireLoader
from conans.client.graph.range_resolver import RangeResolver
from conans.client.installer import BinaryInstaller
from conans.client.loader import ConanFileLoader
from conans.client.recorder.action_recorder import ActionRecorder
from conans.model.graph_info import GraphInfo
from conans.model.manifest import FileTreeManifest
from conans.model.options import OptionsValues
from conans.model.profile import Profile
from conans.model.ref import ConanFileReference
from conans.test.unittests.model.transitive_reqs_test import MockRemoteManager
from conans.test.utils.test_files import temp_folder
from conans.test.utils.tools import GenConanfile
from conans.test.utils.mocks import TestBufferConanOutput
from conans.util.files import save


class GraphManagerTest(unittest.TestCase):

    def setUp(self):
        self.output = TestBufferConanOutput()
        cache_folder = temp_folder()
        cache = ClientCache(cache_folder, self.output)
        self.cache = cache

    def _get_app(self):
        self.remote_manager = MockRemoteManager()
        cache = self.cache
        self.resolver = RangeResolver(self.cache, self.remote_manager)
        proxy = ConanProxy(cache, self.output, self.remote_manager)
        pyreq_loader = PyRequireLoader(proxy, self.resolver)
        pyreq_loader.enable_remotes(remotes=Remotes())
        self.loader = ConanFileLoader(None, self.output, ConanPythonRequire(None, None),
                                      pyreq_loader=pyreq_loader)
        binaries = GraphBinariesAnalyzer(cache, self.output, self.remote_manager)
        self.manager = GraphManager(self.output, cache, self.remote_manager, self.loader, proxy,
                                    self.resolver, binaries)
        generator_manager = GeneratorManager()
        hook_manager = Mock()
        app_type = namedtuple("ConanApp", "cache out remote_manager hook_manager graph_manager"
                              " binaries_analyzer generator_manager")
        app = app_type(self.cache, self.output, self.remote_manager, hook_manager, self.manager,
                       binaries, generator_manager)
        return app

    def recipe_cache(self, reference, requires=None):
        ref = ConanFileReference.loads(reference)
        conanfile = GenConanfile()
        if requires:
            for r in requires:
                conanfile.with_require(r)
        conanfile.with_package_info(
            cpp_info={"libs": ["mylib{}{}lib".format(ref.name, ref.version)]},
            env_info={"MYENV": ["myenv{}{}env".format(ref.name, ref.version)]})
        self._put_in_cache(ref, conanfile)

    def _put_in_cache(self, ref, conanfile):
        layout = self.cache.package_layout(ref)
        save(layout.conanfile(), str(conanfile))
        # Need to complete de metadata = revision + manifest
        with layout.update_metadata() as metadata:
            metadata.recipe.revision = "123"
        manifest = FileTreeManifest.create(layout.export())
        manifest.save(layout.export())

    def alias_cache(self, alias, target):
        ref = ConanFileReference.loads(alias)
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Alias(ConanFile):
                alias = "%s"
            """ % target)
        self._put_in_cache(ref, conanfile)

    @staticmethod
    def recipe_consumer(reference=None, requires=None, build_requires=None):
        path = temp_folder()
        path = os.path.join(path, "conanfile.py")
        conanfile = GenConanfile()
        if reference:
            ref = ConanFileReference.loads(reference)
            conanfile.with_name(ref.name).with_version(ref.version)
        if requires:
            for r in requires:
                conanfile.with_require(r)
        if build_requires:
            for r in build_requires:
                conanfile.with_build_requires(r)
        save(path, str(conanfile))
        return path

    def _cache_recipe(self, ref, test_conanfile, revision=None):
        if isinstance(test_conanfile, GenConanfile):
            name, version = test_conanfile._name, test_conanfile._version
            test_conanfile = test_conanfile.with_package_info(
                cpp_info={"libs": ["mylib{}{}lib".format(name, version)]},
                env_info={"MYENV": ["myenv{}{}env".format(name, version)]})
        save(self.cache.package_layout(ref).conanfile(), str(test_conanfile))
        with self.cache.package_layout(ref).update_metadata() as metadata:
            metadata.recipe.revision = revision or "123"
        manifest = FileTreeManifest.create(self.cache.package_layout(ref).export())
        manifest.save(self.cache.package_layout(ref).export())

    def build_graph(self, content, profile_build_requires=None, ref=None, create_ref=None,
                    install=True):
        path = temp_folder()
        path = os.path.join(path, "conanfile.py")
        save(path, str(content))
        return self.build_consumer(path, profile_build_requires, ref, create_ref, install)

    def build_consumer(self, path, profile_build_requires=None, ref=None, create_ref=None,
                       install=True):
        profile = Profile()
        if profile_build_requires:
            profile.build_requires = profile_build_requires
        profile.process_settings(self.cache)
        update = check_updates = False
        recorder = ActionRecorder()
        remotes = Remotes()
        build_mode = []  # Means build all
        ref = ref or ConanFileReference(None, None, None, None, validate=False)
        options = OptionsValues()
        graph_info = GraphInfo(profile, options=options, root_ref=ref)
        app = self._get_app()
        deps_graph = app.graph_manager.load_graph(path, create_ref, graph_info, build_mode,
                                                  check_updates, update, remotes, recorder)
        if install:
            binary_installer = BinaryInstaller(app, recorder)
            build_mode = BuildMode(build_mode, app.out)
            binary_installer.install(deps_graph, None, build_mode, update, profile_host=profile,
                                     profile_build=None, graph_lock=None)
        return deps_graph

    def _check_node(self, node, ref, deps=None, build_deps=None, dependents=None, closure=None):
        build_deps = build_deps or []
        dependents = dependents or []
        closure = closure or []
        deps = deps or []

        conanfile = node.conanfile
        ref = ConanFileReference.loads(str(ref))
        self.assertEqual(repr(node.ref), repr(ref))
        self.assertEqual(conanfile.name, ref.name)
        self.assertEqual(len(node.dependencies), len(deps) + len(build_deps))

        dependants = node.inverse_neighbors()
        self.assertEqual(len(dependants), len(dependents))
        for d in dependents:
            self.assertIn(d, dependants)

        # The recipe requires is resolved to the reference WITH revision!
        self.assertEqual(len(deps), len(conanfile.requires))
        for dep in deps:
            self.assertEqual(conanfile.requires[dep.name].ref, dep.ref)

        self.assertEqual(closure, list(node.public_closure))
        libs = []
        envs = []
        for n in closure:
            libs.append("mylib%s%slib" % (n.ref.name, n.ref.version))
            envs.append("myenv%s%senv" % (n.ref.name, n.ref.version))
        self.assertListEqual(list(conanfile.deps_cpp_info.libs), libs)
        env = {"MYENV": envs} if envs else {}
        self.assertEqual(conanfile.deps_env_info.vars, env)



import textwrap

import pytest

from conans.test.utils.tools import TestClient


@pytest.fixture
def client():
    c = TestClient()
    conanfile = textwrap.dedent('''
    from conans import ConanFile
    from conans.tools import save, chdir
    import os

    class Protobuf(ConanFile):
        settings = "build_type", "os", "arch", "compiler"

        def package(self):
            my_cmake_module = """
                  function(foo_generate)
                     write_file(foo_generated.h "int from_context = %s;")
                  endfunction()
            """

            with chdir(self.package_folder):
                save("include_build/protobuf.h", "int protubuff_stuff(){ return 1; }")
                save("include_host/protobuf.h", "int protubuff_stuff(){ return 2; }")
                save("build/my_tools_build.cmake", my_cmake_module % "1")
                save("build/my_tools_host.cmake", my_cmake_module % "2")

        def package_info(self):
            # This info depends on self.context !!
            self.cpp_info.includedirs = ["include_{}".format(self.context)]
            path_build_modules = os.path.join("build", "my_tools_{}.cmake".format(self.context))
            self.cpp_info.set_property("cmake_build_modules", [path_build_modules])

    ''')
    c.save({"conanfile.py": conanfile})
    c.run("create . protobuf/1.0@")
    return c


main = textwrap.dedent("""
    #include <iostream>
    #include "protobuf.h"
    #include "foo_generated.h"


    int main(){
        int ret = protubuff_stuff();

        if(ret == 1){
            std::cout << " Library from build context!" << std::endl;
        }
        else if(ret == 2){
            std::cout << " Library from host context!" << std::endl;
        }

        // Variable declared at the foo_generated
        if(from_context == 1){
            std::cout << " Generated code in build context!" << std::endl;
        }
        else if(from_context == 2){
            std::cout << " Generated code in host context!" << std::endl;
        }
        return 0;
    }
    """)

consumer_conanfile = textwrap.dedent("""
    import os
    from conans import ConanFile
    from conan.tools.cmake import CMake, CMakeToolchain, CMakeDeps

    class Consumer(ConanFile):
        settings = "build_type", "os", "arch", "compiler"
        exports_sources = "CMakeLists.txt", "main.cpp"
        requires = "protobuf/1.0"
        build_requires = "protobuf/1.0"

        def generate(self):
            toolchain = CMakeToolchain(self)
            toolchain.generate()

            deps = CMakeDeps(self)
            {}
            deps.generate()

        def build(self):
            cmake = CMake(self)
            cmake.configure()
            cmake.build()
            folder = str(self.settings.build_type) if self.settings.os == "Windows" else "."
            self.run(os.sep.join([folder, "app"]))
    """)


def test_build_modules_from_build_context(client):
    consumer_cmake = textwrap.dedent("""
        set(CMAKE_CXX_COMPILER_WORKS 1)
        set(CMAKE_CXX_ABI_COMPILED 1)
        cmake_minimum_required(VERSION 3.15)
        project(MyApp CXX)

        find_package(protobuf)
        find_package(protobuf_BUILD)
        add_executable(app main.cpp)
        foo_generate()
        target_link_libraries(app protobuf::protobuf)
        """)

    cmake_deps_conf = """
        deps.build_context_activated = ["protobuf"]
        deps.build_context_build_modules = ["protobuf"]
        deps.build_context_suffix = {"protobuf": "_BUILD"}
    """

    client.save({"conanfile.py": consumer_conanfile.format(cmake_deps_conf),
                 "CMakeLists.txt": consumer_cmake,
                 "main.cpp": main})

    client.run("create . app/1.0@ -pr:b default -pr:h default")
    assert "Library from host context!" in client.out
    assert "Generated code in build context!" in client.out


def test_build_modules_and_target_from_build_context(client):
    consumer_cmake = textwrap.dedent("""
        set(CMAKE_CXX_COMPILER_WORKS 1)
        set(CMAKE_CXX_ABI_COMPILED 1)
        cmake_minimum_required(VERSION 3.15)
        project(MyApp CXX)

        find_package(protobuf)
        find_package(protobuf_BUILD)
        add_executable(app main.cpp)
        foo_generate()
        target_link_libraries(app protobuf_BUILD::protobuf_BUILD)
        """)

    cmake_deps_conf = """
        deps.build_context_activated = ["protobuf"]
        deps.build_context_build_modules = ["protobuf"]
        deps.build_context_suffix = {"protobuf": "_BUILD"}
    """

    client.save({"conanfile.py": consumer_conanfile.format(cmake_deps_conf),
                 "CMakeLists.txt": consumer_cmake,
                 "main.cpp": main})

    client.run("create . app/1.0@ -pr:b default -pr:h default")
    assert "Library from build context!" in client.out
    assert "Generated code in build context!" in client.out


def test_build_modules_from_host_and_target_from_build_context(client):
    consumer_cmake = textwrap.dedent("""
        set(CMAKE_CXX_COMPILER_WORKS 1)
        set(CMAKE_CXX_ABI_COMPILED 1)
        cmake_minimum_required(VERSION 3.15)
        project(MyApp CXX)

        find_package(protobuf)
        find_package(protobuf_BUILD)
        add_executable(app main.cpp)
        foo_generate()
        target_link_libraries(app protobuf_BUILD::protobuf_BUILD)
        """)

    cmake_deps_conf = """
        deps.build_context_activated = ["protobuf"]
        deps.build_context_suffix = {"protobuf": "_BUILD"}
    """

    client.save({"conanfile.py": consumer_conanfile.format(cmake_deps_conf),
                 "CMakeLists.txt": consumer_cmake,
                 "main.cpp": main})

    client.run("create . app/1.0@ -pr:b default -pr:h default")
    assert "Library from build context!" in client.out
    assert "Generated code in host context!" in client.out


def test_build_modules_and_target_from_host_context(client):
    consumer_cmake = textwrap.dedent("""
        set(CMAKE_CXX_COMPILER_WORKS 1)
        set(CMAKE_CXX_ABI_COMPILED 1)
        cmake_minimum_required(VERSION 3.15)
        project(MyApp CXX)

        find_package(protobuf)
        find_package(protobuf_BUILD)
        add_executable(app main.cpp)
        foo_generate()
        target_link_libraries(app protobuf::protobuf)
        """)

    cmake_deps_conf = """
        deps.build_context_activated = ["protobuf"]
        deps.build_context_build_modules = []
        deps.build_context_suffix = {"protobuf": "_BUILD"}
    """

    client.save({"conanfile.py": consumer_conanfile.format(cmake_deps_conf),
                 "CMakeLists.txt": consumer_cmake,
                 "main.cpp": main})

    client.run("create . app/1.0@ -pr:b default -pr:h default")
    assert "Conan: Target declared 'protobuf::protobuf'" in client.out
    assert "Conan: Target declared 'protobuf_BUILD::protobuf_BUILD'" in client.out
    assert "Library from host context!" in client.out
    assert "Generated code in host context!" in client.out


def test_exception_when_not_prefix_specified(client):
    cmake_deps_conf = """
        deps.build_context_activated = ["protobuf"]
    """
    client.save({"conanfile.py": consumer_conanfile.format(cmake_deps_conf),
                 "main.cpp": main})

    client.run("create . app/1.0@ -pr:b default -pr:h default", assert_error=True)
    assert "The package 'protobuf' exists both as 'require' and as 'build require'. " \
           "You need to specify a suffix using the 'build_context_suffix' attribute at the " \
           "CMakeDeps generator." in client.out


def test_not_activated_not_fail(client):
    consumer_cmake = textwrap.dedent("""
        set(CMAKE_CXX_COMPILER_WORKS 1)
        set(CMAKE_CXX_ABI_COMPILED 1)
        cmake_minimum_required(VERSION 3.15)
        project(MyApp CXX)

        find_package(protobuf)
        add_executable(app main.cpp)
        foo_generate()
        target_link_libraries(app protobuf::protobuf)
        """)

    client.save({"conanfile.py": consumer_conanfile.format(""),
                 "CMakeLists.txt": consumer_cmake,
                 "main.cpp": main})

    client.run("create . app/1.0@ -pr:b default -pr:h default")
    assert "app/1.0: Created package" in client.out
    assert "Library from host context!" in client.out
    assert "Generated code in host context!" in client.out



import textwrap

from conans.client.graph.graph import CONTEXT_BUILD, CONTEXT_HOST
from conans.model.profile import Profile
from conans.test.integration.graph.core.cross_build._base_test_case import CrossBuildingBaseTestCase


class BuildRequireOfBuildRequire(CrossBuildingBaseTestCase):
    """ There is an application that build_requires three different tools:
         * cmake (build)
         * gtest (host), that build_requires 'cmake'
         * protoc (build), that build_requires 'cmake' too
    """

    application = textwrap.dedent("""
        from conans import ConanFile

        class Application(ConanFile):
            name = "app"
            version = "testing"

            settings = "os"

            def build_requirements(self):
                self.build_requires("cmake/testing@user/channel")
                self.build_requires("gtest/testing@user/channel", force_host_context=True)
                self.build_requires("protoc/testing@user/channel")

            def build(self):
                self.output.info(">> settings.os:".format(self.settings.os))
    """)

    gtest = CrossBuildingBaseTestCase.gtest_tpl.render(
        build_requires=((CrossBuildingBaseTestCase.cmake_ref, False), ))
    protoc = CrossBuildingBaseTestCase.protoc_tpl.render(
        build_requires=((CrossBuildingBaseTestCase.cmake_ref, False), ))

    def setUp(self):
        super(BuildRequireOfBuildRequire, self).setUp()
        self._cache_recipe(self.cmake_ref, self.cmake)
        self._cache_recipe(self.gtest_ref, self.gtest)
        self._cache_recipe(self.protobuf_ref, self.protobuf)
        self._cache_recipe(self.protoc_ref, self.protoc)
        self._cache_recipe(self.app_ref, self.application)

    def test_crossbuilding(self):
        profile_host = Profile()
        profile_host.settings["os"] = "Host"
        profile_host.process_settings(self.cache)

        profile_build = Profile()
        profile_build.settings["os"] = "Build"
        profile_build.process_settings(self.cache)

        deps_graph = self._build_graph(profile_host=profile_host, profile_build=profile_build,
                                       install=True)

        # Check HOST packages
        #   - Application
        application = deps_graph.root.dependencies[0].dst
        self.assertEqual(len(application.dependencies), 3)
        self.assertEqual(application.conanfile.name, "app")
        self.assertEqual(application.context, CONTEXT_HOST)
        self.assertEqual(application.conanfile.settings.os, profile_host.settings['os'])

        #   - Application::deps_cpp_info:
        gtest_cpp_info = application.conanfile.deps_cpp_info["gtest"]
        self.assertEqual(gtest_cpp_info.includedirs, ['gtest-host'])
        self.assertEqual(gtest_cpp_info.libdirs, ['gtest-host'])
        self.assertEqual(gtest_cpp_info.bindirs, ['gtest-host'])

        with self.assertRaises(KeyError):
            _ = application.conanfile.deps_cpp_info["cmake"]

        with self.assertRaises(KeyError):
            _ = application.conanfile.deps_cpp_info["protoc"]

        #   - Application::deps_env_info:
        cmake_env_info = application.conanfile.deps_env_info["cmake"]
        self.assertEqual(cmake_env_info.PATH, ['cmake-build'])
        self.assertEqual(cmake_env_info.OTHERVAR, 'cmake-build')

        protoc_env_info = application.conanfile.deps_env_info["protoc"]
        self.assertEqual(protoc_env_info.PATH, ['protoc-build'])
        self.assertEqual(protoc_env_info.OTHERVAR, 'protoc-build')

        with self.assertRaises(KeyError):
            _ = application.conanfile.deps_env_info["gtest"]

        #   - GTest
        gtest_host = application.dependencies[2].dst
        self.assertEqual(gtest_host.conanfile.name, "gtest")
        self.assertEqual(gtest_host.context, CONTEXT_HOST)
        self.assertEqual(gtest_host.conanfile.settings.os, profile_host.settings['os'])

        #   - GTest::deps_cpp_info:
        with self.assertRaises(KeyError):
            _ = gtest_host.conanfile.deps_cpp_info["cmake"]

        #   - GTest::deps_env_info:
        cmake_env_info = gtest_host.conanfile.deps_env_info["cmake"]
        self.assertEqual(cmake_env_info.PATH, ['cmake-build'])
        self.assertEqual(cmake_env_info.OTHERVAR, 'cmake-build')

        # Check BUILD packages
        #   - CMake
        cmake_build = application.dependencies[0].dst
        self.assertEqual(cmake_build.conanfile.name, "cmake")
        self.assertEqual(cmake_build.context, CONTEXT_BUILD)
        self.assertEqual(str(cmake_build.conanfile.settings.os), profile_build.settings['os'])

        #   - Protoc
        protoc_build = application.dependencies[1].dst
        self.assertEqual(protoc_build.conanfile.name, "protoc")
        self.assertEqual(protoc_build.context, CONTEXT_BUILD)
        self.assertEqual(str(protoc_build.conanfile.settings.os), profile_build.settings['os'])

        #   - Protoc::deps_cpp_info:
        with self.assertRaises(KeyError):
            _ = protoc_build.conanfile.deps_cpp_info["cmake"]

        #   - Protoc::deps_env_info:
        cmake_env_info = protoc_build.conanfile.deps_env_info["cmake"]
        self.assertEqual(cmake_env_info.PATH, ['cmake-build'])
        self.assertEqual(cmake_env_info.OTHERVAR, 'cmake-build')

        #   - CMake <- Protoc
        cmake_protoc_build = protoc_build.dependencies[1].dst
        self.assertNotEqual(cmake_build, cmake_protoc_build)
        self.assertEqual(cmake_protoc_build.conanfile.name, "cmake")
        self.assertEqual(cmake_protoc_build.context, CONTEXT_BUILD)
        self.assertEqual(str(cmake_protoc_build.conanfile.settings.os), profile_build.settings['os'])

        #   - CMake <- GTest
        cmake_gtest_build = gtest_host.dependencies[0].dst
        self.assertNotEqual(cmake_build, cmake_gtest_build)
        self.assertEqual(cmake_gtest_build.conanfile.name, "cmake")
        self.assertEqual(cmake_gtest_build.context, CONTEXT_BUILD)
        self.assertEqual(str(cmake_gtest_build.conanfile.settings.os), profile_build.settings['os'])



import platform
import textwrap

import pytest

from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient


@pytest.fixture(scope="module")
def client():
    openssl = textwrap.dedent(r"""
        import os
        from conans import ConanFile
        from conans.tools import save, chdir
        class Pkg(ConanFile):
            settings = "os"
            def package(self):
                with chdir(self.package_folder):
                    echo = "@echo off\necho MYOPENSSL={}!!".format(self.settings.os)
                    save("bin/myopenssl.bat", echo)
                    save("bin/myopenssl.sh", echo)
                    os.chmod("bin/myopenssl.sh", 0o777)

            def package_info(self):
                self.env_info.PATH = [os.path.join(self.package_folder, "bin")]
            """)

    cmake = textwrap.dedent(r"""
        import os
        from conans import ConanFile
        from conans.tools import save, chdir
        class Pkg(ConanFile):
            settings = "os"
            requires = "openssl/1.0"
            def package(self):
                with chdir(self.package_folder):
                    echo = "@echo off\necho MYCMAKE={}!!".format(self.settings.os)
                    save("mycmake.bat", echo + "\ncall myopenssl.bat")
                    save("mycmake.sh", echo + "\n myopenssl.sh")
                    os.chmod("mycmake.sh", 0o777)

            def package_info(self):
                self.env_info.PATH = [self.package_folder]
            """)

    client = TestClient()
    client.save({"tool/conanfile.py": GenConanfile(),
                 "cmake/conanfile.py": cmake,
                 "openssl/conanfile.py": openssl})

    client.run("create tool tool/1.0@")
    client.run("create openssl openssl/1.0@")
    client.run("create cmake mycmake/1.0@")
    return client


@pytest.mark.parametrize("existing_br", ["",
                                         'build_requires="tool/1.0"',
                                         'build_requires=("tool/1.0", )',
                                         'build_requires=["tool/1.0"]'])
@pytest.mark.parametrize("build_profile", ["", "-pr:b=default"])
def test_build_require_test_package(existing_br, build_profile, client):
    test_cmake = textwrap.dedent(r"""
        import os, platform
        from conans import ConanFile
        from conans.tools import save, chdir
        class Pkg(ConanFile):
            settings = "os"
            test_type = "build_requires"
            {}

            def build(self):
                mybuild_cmd = "mycmake.bat" if platform.system() == "Windows" else "mycmake.sh"
                self.run(mybuild_cmd)

            def test(self):
                pass
        """)

    # Test with extra build_requires to check it doesn't interfere or get deleted
    client.save({"cmake/test_package/conanfile.py": test_cmake.format(existing_br)})
    client.run("create cmake mycmake/1.0@ {} --build=missing".format(build_profile))

    def check(out):
        if "tool" in existing_br:
            assert "mycmake/1.0 (test package): Applying build-requirement: tool/1.0" in out
        else:
            assert "tool/1.0" not in out

        assert "mycmake/1.0 (test package): Applying build-requirement: openssl/1.0" in out
        assert "mycmake/1.0 (test package): Applying build-requirement: mycmake/1.0" in out

        system = {"Darwin": "Macos"}.get(platform.system(), platform.system())
        assert "MYCMAKE={}!!".format(system) in out
        assert "MYOPENSSL={}!!".format(system) in out

    check(client.out)

    client.run("test cmake/test_package mycmake/1.0@ {}".format(build_profile))
    check(client.out)


@pytest.mark.parametrize("existing_br", ["",
                                         'build_requires="tool/1.0"',
                                         'build_requires=("tool/1.0", )',
                                         'build_requires=["tool/1.0"]'])
def test_both_types(existing_br, client):
    # When testing same package in both contexts, the 2 profiles approach must be used
    test_cmake = textwrap.dedent(r"""
        import os, platform
        from conans import ConanFile
        from conans.tools import save, chdir
        class Pkg(ConanFile):
            settings = "os"
            test_type = "build_requires", "requires"
            {}

            def build(self):
                mybuild_cmd = "mycmake.bat" if platform.system() == "Windows" else "mycmake.sh"
                self.run(mybuild_cmd)

            def test(self):
                pass
        """)

    # Test with extra build_requires to check it doesn't interfere or get deleted
    client.save({"cmake/test_package/conanfile.py": test_cmake.format(existing_br)})
    # This must use the build-host contexts to have same dep in different contexts
    client.run("create cmake mycmake/1.0@ -pr:b=default --build=missing")

    def check(out):
        if "tool" in existing_br:
            assert "mycmake/1.0 (test package): Applying build-requirement: tool/1.0" in out
        else:
            assert "tool/1.0" not in out

        assert "mycmake/1.0 (test package): Applying build-requirement: openssl/1.0" in out
        assert "mycmake/1.0 (test package): Applying build-requirement: mycmake/1.0" in out

        system = {"Darwin": "Macos"}.get(platform.system(), platform.system())
        assert "MYCMAKE={}!!".format(system) in out
        assert "MYOPENSSL={}!!".format(system) in out

    check(client.out)

    client.run("test cmake/test_package mycmake/1.0@ -pr:b=default")
    check(client.out)


def test_create_build_requires():
    # test that I can create a package passing the build and host context and package will get both
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class Pkg(ConanFile):
            settings = "os"

            def package_info(self):
                self.output.info("MYOS=%s!!!" % self.settings.os)
                self.output.info("MYTARGET={}!!!".format(self.settings_target.os))
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . br/0.1@  --build-require -s:h os=Linux -s:b os=Windows")
    assert "br/0.1:3475bd55b91ae904ac96fde0f106a136ab951a5e" in client.out
    assert "br/0.1:cb054d0b3e1ca595dc66bc2339d40f1f8f04ab31" not in client.out
    assert "br/0.1: MYOS=Windows!!!" in client.out
    assert "br/0.1: MYTARGET=Linux!!!" in client.out
    assert "br/0.1: MYOS=Linux!!!" not in client.out


def test_build_require_conanfile_text(client):
    client.save({"conanfile.txt": "[build_requires]\nmycmake/1.0"}, clean_first=True)
    client.run("install . -g virtualenv")
    cmd = ". ./activate.sh && mycmake.sh" if platform.system() != "Windows" else \
        "activate.bat && mycmake.bat"
    client.run_command(cmd)
    system = {"Darwin": "Macos"}.get(platform.system(), platform.system())
    assert "MYCMAKE={}!!".format(system) in client.out
    assert "MYOPENSSL={}!!".format(system) in client.out


def test_build_require_command_line_no_context(client):
    client.run("install mycmake/1.0@  -g virtualenv")
    cmd = ". ./activate.sh && mycmake.sh" if platform.system() != "Windows" else \
        "activate.bat && mycmake.bat"
    client.run_command(cmd)
    system = {"Darwin": "Macos"}.get(platform.system(), platform.system())
    assert "MYCMAKE={}!!".format(system) in client.out
    assert "MYOPENSSL={}!!".format(system) in client.out


def test_build_require_command_line_build_context(client):
    client.run("install mycmake/1.0@ --build-require -g virtualenv -pr:b=default")
    cmd = ". ./activate.sh && mycmake.sh" if platform.system() != "Windows" else \
        "activate.bat && mycmake.bat"
    client.run_command(cmd)
    system = {"Darwin": "Macos"}.get(platform.system(), platform.system())
    assert "MYCMAKE={}!!".format(system) in client.out
    assert "MYOPENSSL={}!!".format(system) in client.out



import platform
import textwrap
import unittest
import pytest

from conans.test.utils.tools import TestClient


class MesonTest(unittest.TestCase):

    @pytest.mark.skipif(platform.system() != "Windows", reason="Needs windows for vcvars")
    @pytest.mark.tool_visual_studio
    def test_vcvars_priority(self):
        # https://github.com/conan-io/conan/issues/5999
        client = TestClient()
        conanfile_vcvars = textwrap.dedent("""
            import os
            from conans import ConanFile, Meson

            class HelloConan(ConanFile):
                settings = "os", "compiler", "arch", "build_type"
                def build(self):
                    meson = Meson(self, append_vcvars=True)
                    meson.configure()

                # CAPTURING THE RUN METHOD
                def run(self, cmd):
                    self.output.info("PATH ENV VAR: %s" % os.getenv("PATH"))
            """)

        client.save({"conanfile.py": conanfile_vcvars})
        client.run('create . pkg/1.0@ -e PATH="MyCustomPath"')
        self.assertIn("pkg/1.0: PATH ENV VAR: MyCustomPath;", client.out)



import textwrap

import pytest

from conans.test.assets.cmake import gen_cmakelists
from conans.test.assets.sources import gen_function_cpp
from conans.test.utils.tools import TestClient


def test_exports_sources_own_code_in_subfolder():
    """ test that we can put the conanfile in a subfolder, and it can work. The key is
    the exports_sources() method that can do:
        os.path.join(self.recipe_folder, "..")
    And the layout: self.folders.root = ".."
    """
    c = TestClient()
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import load, copy, save
        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"

            def layout(self):
                self.folders.root = ".."
                self.folders.source = "."
                self.folders.build = "build"
                self.folders.generators = "conangen"

            def generate(self):
                save(self, "mygen.txt", "mycontent")

            def export_sources(self):
                source_folder = os.path.join(self.recipe_folder, "..")
                copy(self, "*.txt", source_folder, self.export_sources_folder)

            def source(self):
                cmake = load(self, "CMakeLists.txt")
                self.output.info("MYCMAKE-SRC: {}".format(cmake))

            def build(self):
                path = os.path.join(self.source_folder, "CMakeLists.txt")
                cmake = load(self, path)
                self.output.info("MYCMAKE-BUILD: {}".format(cmake))
                save(self, "mylib.a", "mylib")
            """)
    c.save({"conan/conanfile.py": conanfile,
            "CMakeLists.txt": "mycmake!"})
    c.run("create conan")
    assert "pkg/0.1: MYCMAKE-SRC: mycmake!" in c.out
    assert "pkg/0.1: MYCMAKE-BUILD: mycmake!" in c.out

    # Local flow
    c.run("install conan")
    assert c.load("conangen/mygen.txt") == "mycontent"
    # SOURCE NOT CALLED! It doesnt make sense (will fail due to local exports)
    c.run("build conan")
    assert "conanfile.py (pkg/0.1): MYCMAKE-BUILD: mycmake!" in c.out
    assert c.load("build/mylib.a") == "mylib"


def test_exports_sources_common_code():
    """ very similar to the above, but intended for a multi-package project sharing some
    common code
    """
    c = TestClient()
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import load, copy, save
        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"

            def layout(self):
                self.folders.root = ".."
                self.folders.source = "pkg"
                self.folders.build = "build"

            def export_sources(self):
                source_folder = os.path.join(self.recipe_folder, "..")
                copy(self, "*.txt", source_folder, self.export_sources_folder)
                copy(self, "*.cmake", source_folder, self.export_sources_folder)

            def source(self):
                cmake = load(self, "CMakeLists.txt")
                self.output.info("MYCMAKE-SRC: {}".format(cmake))

            def build(self):
                path = os.path.join(self.source_folder, "CMakeLists.txt")
                cmake = load(self, path)
                self.output.info("MYCMAKE-BUILD: {}".format(cmake))

                path = os.path.join(self.export_sources_folder, "common", "myutils.cmake")
                cmake = load(self, path)
                self.output.info("MYUTILS-BUILD: {}".format(cmake))
            """)
    c.save({"pkg/conanfile.py": conanfile,
            "pkg/CMakeLists.txt": "mycmake!",
            "common/myutils.cmake": "myutils!"})
    c.run("create pkg")
    assert "pkg/0.1: MYCMAKE-SRC: mycmake!" in c.out
    assert "pkg/0.1: MYCMAKE-BUILD: mycmake!" in c.out
    assert "pkg/0.1: MYUTILS-BUILD: myutils!" in c.out

    # Local flow
    c.run("install pkg")
    # SOURCE NOT CALLED! It doesnt make sense (will fail due to local exports)
    c.run("build pkg")
    assert "conanfile.py (pkg/0.1): MYCMAKE-BUILD: mycmake!" in c.out
    assert "conanfile.py (pkg/0.1): MYUTILS-BUILD: myutils!" in c.out


@pytest.mark.tool_cmake
def test_exports_sources_common_code_layout():
    """ Equal to the previous test, but actually building and using cmake_layout
    """
    c = TestClient()
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.cmake import cmake_layout, CMake
        from conan.tools.files import load, copy, save
        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"
            settings = "os", "compiler", "build_type", "arch"
            generators = "CMakeToolchain"

            def layout(self):
                self.folders.root = ".."
                self.folders.subproject = "pkg"
                cmake_layout(self)

            def export_sources(self):
                source_folder = os.path.join(self.recipe_folder, "..")
                copy(self, "*", source_folder, self.export_sources_folder)

            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()
                self.run(os.path.join(self.cpp.build.bindirs[0], "myapp"))
            """)
    cmake_include = "include(${CMAKE_CURRENT_LIST_DIR}/../common/myutils.cmake)"
    c.save({"pkg/conanfile.py": conanfile,
            "pkg/app.cpp": gen_function_cpp(name="main", includes=["../common/myheader"],
                                            preprocessor=["MYDEFINE"]),
            "pkg/CMakeLists.txt": gen_cmakelists(appsources=["app.cpp"],
                                                 custom_content=cmake_include),
            "common/myutils.cmake": 'message(STATUS "MYUTILS.CMAKE!")',
            "common/myheader.h": '#define MYDEFINE "MYDEFINEVALUE"'})
    c.run("create pkg")
    assert "MYUTILS.CMAKE!" in c.out
    assert "main: Release!" in c.out
    assert "MYDEFINE: MYDEFINEVALUE" in c.out

    # Local flow
    c.run("install pkg")
    c.run("build pkg")
    assert "MYUTILS.CMAKE!" in c.out
    assert "main: Release!" in c.out
    assert "MYDEFINE: MYDEFINEVALUE" in c.out

    c.run("install pkg -s build_type=Debug")
    c.run("build pkg")
    assert "MYUTILS.CMAKE!" in c.out
    assert "main: Debug!" in c.out
    assert "MYDEFINE: MYDEFINEVALUE" in c.out



# coding=utf-8


import unittest

import six

from conans.client.tools.version import Version
from conans.errors import ConanException


class ToolVersionMainComponentsTests(unittest.TestCase):

    def test_invalid_values(self):
        self.assertRaises(ConanException, Version, "")
        self.assertRaises(ConanException, Version, "nonsense")
        self.assertRaises(ConanException, Version, "a1.2.3")

    def test_invalid_message(self):
        with six.assertRaisesRegex(self, ConanException, "Invalid version 'not-valid'"):
            Version("not-valid")

    def test_valid_values(self):
        for v_str in ["1.2.3", "1.2.3-dev90", "1.2.3+dev90", "1.2.3-dev90+more", "1.2.3-dev90+a.b"]:
            v = Version(v_str)
            self.assertEqual(v.major, "1")
            self.assertEqual(v.minor, "2")
            self.assertEqual(v.patch, "3")

    def test_valid_loose(self):
        self.assertTrue(Version.loose)

        # These versions are considered valid with loose validation
        self.assertTrue(Version.loose)
        v = Version("1.2")
        self.assertEqual(v.major, "1")
        self.assertEqual(v.minor, "2")
        self.assertEqual(v.patch, "0")

        v = Version("1")
        self.assertEqual(v.major, "1")
        self.assertEqual(v.minor, "0")
        self.assertEqual(v.patch, "0")

        v = Version(" 1a")
        self.assertEqual(v.major, "1")
        self.assertEqual(v.minor, "0")
        self.assertEqual(v.patch, "0")

    def test_convert_str(self):
        # Check that we are calling the string method
        class A(object):
            def __str__(self):
                return "1.2.3"

        v = Version(A())
        self.assertEqual(v.major, "1")
        self.assertEqual(v.minor, "2")
        self.assertEqual(v.patch, "3")

    def test_compare(self):
        self.assertTrue(Version("1.2.3") == "1.2.3")
        self.assertTrue(Version("1.2.3") == Version("1.2.3"))

        self.assertTrue(Version.loose)
        self.assertTrue(Version("234") == "234")
        self.assertTrue(Version("234") == Version("234"))

    def test_gt(self):
        self.assertTrue(Version("1.2.3") > "1.2.2")

        self.assertTrue(Version.loose)
        self.assertTrue(Version("1.2") > "1")
        self.assertTrue(Version("1.2") > Version("1"))

        self.assertFalse(Version("1.0") > "1")
        self.assertFalse(Version("1.0.0") > "1")
        self.assertFalse(Version("1") > "1.0")
        self.assertFalse(Version("1") > "1.0.0")


class ToolVersionExtraComponentsTests(unittest.TestCase):

    def test_parsing(self):
        v = Version("1.2.3-dev.90.80-dev2+build.b1.b2")
        self.assertEqual(v.prerelease, "dev.90.80-dev2")
        self.assertEqual(v.build, "build.b1.b2")

        v = Version("1.2.305-dev4")
        self.assertEqual(v.prerelease, "dev4")
        self.assertEqual(v.build, "")

        v = Version("1.2.305+dev4")
        self.assertEqual(v.prerelease, "")
        self.assertEqual(v.build, "dev4")

    def test_compare(self):
        # prerelease is taken into account
        self.assertTrue(Version("1.2.3-dev90") != "1.2.3")
        self.assertTrue(Version("1.2.3-dev90") < "1.2.3")
        self.assertTrue(Version("1.2.3-dev90") < Version("1.2.3"))

        # build is not taken into account
        self.assertTrue(Version("1.2") == Version("1.2+dev0"))
        self.assertFalse(Version("1.2") > Version("1.2+dev0"))
        self.assertFalse(Version("1.2") < Version("1.2+dev0"))

        # Unknown release field, not fail (loose=True) and don't affect compare
        self.assertTrue(Version.loose)
        self.assertTrue(Version("1.2.3.4") == Version("1.2.3"))




import os
import platform

import pytest

from conan.tools.files import check_md5, check_sha1, check_sha256, collect_libs
from conans.model.build_info import CppInfo
from conans.test.utils.mocks import ConanFileMock
from conans.test.utils.test_files import temp_folder
from conans.util.files import save


def test_collect_libs():
    conanfile = ConanFileMock()
    conanfile.cpp_info = CppInfo(conanfile.name, "")
    # Without package_folder
    result = collect_libs(conanfile)
    assert [] == result

    # Default behavior
    conanfile.folders.set_base_package(temp_folder())
    mylib_path = os.path.join(conanfile.package_folder, "lib", "mylib.lib")
    save(mylib_path, "")

    result = collect_libs(conanfile)
    assert ["mylib"] == result

    # Custom folder
    customlib_path = os.path.join(conanfile.package_folder, "custom_folder", "customlib.lib")
    save(customlib_path, "")
    result = collect_libs(conanfile, folder="custom_folder")
    assert ["customlib"] == result

    # Custom folder doesn't exist
    result = collect_libs(conanfile, folder="fake_folder")
    assert [] == result
    assert "Lib folder doesn't exist, can't collect libraries:" in conanfile.output

    # Use cpp_info.libdirs
    conanfile.cpp_info.libdirs = ["lib", "custom_folder"]
    result = collect_libs(conanfile)
    assert ["customlib", "mylib"] == result

    # Custom folder with multiple libdirs should only collect from custom folder
    assert ["lib", "custom_folder"] == conanfile.cpp_info.libdirs
    result = collect_libs(conanfile, folder="custom_folder")
    assert ["customlib"] == result

    # Unicity of lib names
    conanfile = ConanFileMock()
    conanfile.folders.set_base_package(temp_folder())
    conanfile.cpp_info = CppInfo(conanfile.name, "")
    custom_mylib_path = os.path.join(conanfile.package_folder, "custom_folder", "mylib.lib")
    lib_mylib_path = os.path.join(conanfile.package_folder, "lib", "mylib.lib")
    save(custom_mylib_path, "")
    save(lib_mylib_path, "")
    conanfile.cpp_info.libdirs = ["lib", "custom_folder"]
    result = collect_libs(conanfile)
    assert ["mylib"] == result

    # Warn lib folder does not exist with correct result
    conanfile = ConanFileMock()
    conanfile.folders.set_base_package(temp_folder())
    conanfile.cpp_info = CppInfo(conanfile.name, "")
    lib_mylib_path = os.path.join(conanfile.package_folder, "lib", "mylib.lib")
    save(lib_mylib_path, "")
    no_folder_path = os.path.join(conanfile.package_folder, "no_folder")
    conanfile.cpp_info.libdirs = ["no_folder", "lib"]  # 'no_folder' does NOT exist
    result = collect_libs(conanfile)
    assert ["mylib"] == result
    assert "WARN: Lib folder doesn't exist, can't collect libraries: %s" % no_folder_path \
           in conanfile.output

@pytest.mark.skipif(platform.system() == "Windows", reason="Needs symlinks support")
def test_collect_libs_symlinks():
    # Keep only the shortest lib name per group of symlinks
    conanfile = ConanFileMock()
    conanfile.folders.set_base_package(temp_folder())
    conanfile.cpp_info = CppInfo(conanfile.name, "")
    version_mylib_path = os.path.join(conanfile.package_folder, "lib", "libmylib.1.0.0.dylib")
    soversion_mylib_path = os.path.join(conanfile.package_folder, "lib", "libmylib.1.dylib")
    lib_mylib_path = os.path.join(conanfile.package_folder, "lib", "libmylib.dylib")
    lib_mylib2_path = os.path.join(conanfile.package_folder, "lib", "libmylib.2.dylib")
    lib_mylib3_path = os.path.join(conanfile.package_folder, "custom_folder", "libmylib.3.dylib")
    save(version_mylib_path, "")
    os.symlink(version_mylib_path, soversion_mylib_path)
    os.symlink(soversion_mylib_path, lib_mylib_path)
    save(lib_mylib2_path, "")
    save(lib_mylib3_path, "")
    conanfile.cpp_info.libdirs = ["lib", "custom_folder"]
    result = collect_libs(conanfile)
    assert ["mylib", "mylib.2", "mylib.3"] == result
